! ===========================================================================================================
! subroutine pick_profile_pos
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20191126 JN     Adjustments to account for 999 lines.
! 20190425 VH     Initialize ice_int_status to zero, not 1
! 2019-416 VH     Complete revision of messaging.  When posloc is set, posn_cmnt is immediaely created to descripbe
!                 the situation. Interpolated osition without ice lines in the phy file will be a warning, not an error
! 20190301 JN     Handle the files with no primary core argo profile, but only secondary profile (from SIO & PMEL)
! 20190111 JN     To determine final prof_POSITION_QC, use the larger of the aoml-determined overall qc
!                 and the PI-supplied qc
! 20181231 JN     Take position_qc from the PI-supplied qc if it is 2 or 3
! 20181218 JN     Exclude interpolated positions in the Traj.nc file
! 20181206 JN     Changed FATAL ERROR 223 (No valid position found) to a WARNING
! 20181127 JN     Initial version
!
! ===========================================================================================================

subroutine pick_profile_pos(caller,pathname_phyfile)

use communication                       ! module   
use data_arrays                         ! module
use ncmeta_arrays                       ! module     

implicit none

character(*), intent( in ) :: caller,pathname_phyfile
logical :: best_position_found		! Did we find good or semi-good position?
! This is used to fine JULD_LOCATION, taking into account DEEP_ARGO FLOATS
integer :: posbeg, posend,posinc;
character(len=20),  parameter ::  sub = 'pick_profile_pos'
integer :: max_dateqc=0,max_posnqc=0,max_posiqc=0,max_speedqc=0;
integer :: test(4) = (/ 1, 2, 3, 4 /);

! ======================================================================================
! Initialize position variables
posloc=0; posn_cmnt="0 Initialize posloc to zero, set best_position_found=F";
ice_int_status=0; ice_int_cmnt = ice_int_cmnts(ice_int_status); !
if(DBGPOS)then; write(DBG,'("DBGPOSLOC: ",a)')trim(posn_cmnt); endif
best_position_found = .false.; 
prof_POSITION_QC = '9'; int_POSITION_QC = 9;
prof_LATITUDE  = FILLPOSN;  real_LATITUDE  = FILL;
prof_LONGITUDE  = FILLPOSN; real_LONGITUDE  = FILL;
prof_DATE_TIME_LOCATION = "99999999999999";
prof_JULD_LOCATION = FILLDBLE;
!========================================================!
! Now that all positions/times have been qc-ed, select the JULD_LOCATION (the first with both good time & good posn.
! First, see if the PI-supplied(if present) is different from out real-time qc, specifically if pisqc=4, posqc=1
! 02apr2015 - for now ACCEPT what John is putting for posiitn qc
! Put a NOTE in the log file, do not issue a warning.
! 30sep2015 - some positions come with a PI-supplied qc of "8", meaning it is interpolated (common for under-ice profiles)
! if out rtpos is 4, or if on land, then the position will receive a "4:, otherwise, retain the "8"
! ======================================================================================
! Just in case we have NO position lines
if(DBLEV==71)then; npos = 0; endif
if(npos == 0)then
    posloc=0; posn_cmnt="0, There are no position lines";
    if(DBGPOS)then; write(DBG,'("DBGPOSLOC: ",a)')trim(posn_cmnt); endif;
    return                    
endif
if(npos2 == 0)then
    if(npos == 1)then
	posloc=0; posn_cmnt="0, There is 1 position line, but it is a 999 line";
    else
	posloc=0; write(posn_cmnt,'("0, There are ",i1," position lines, but they are all 999 lines")')npos;
    endif
    if(DBGPOS)then; write(DBG,'("DBGPOSLOC: ",a)')trim(posn_cmnt); endif;
    return
endif

! ======================================================================================
! Analyze the position block qc flags - note that this only determins the qc flags for
! each position, and do a quick check for interpolated (not the final check)

!need define arrays poslocv and poslocvI as vectors of same length as position block (see data_arrays)

call analyze_positions(sub,ret);

if(ret /= SUCCESS)then
    if(ret == FAILURE)then
	write(msg,'(a,"A NON-FATAL error occurred in analyze_positions error issued there")')trim(sub); call wrtlog(sub,msg);
    else
	if(DEBUG)then
	    write(errmsg(4),'("FATAL ERROR 905 DBG=T profile: ",a)')trim(profile)
	    write(errmsg(3),'("FATAL ERROR 905 DBG=T profile name: ",a)') trim(profinp);
	    write(errmsg(1),'("FATAL ERROR 905 DBG=T a non-standard return value was reported from analyze_positions")')
	    write(errmsg(2),'("FATAL ERROR 905 DBG=T  the value is: ",i3)')ret;
	    call error_exit(sub,905,4,errmsg); ! DBG=T qcplus - non-standard ret value from analyze positions
	else
	    write(errmsg(4),'("NON-FATAL ERROR 205 DBG=F profile: ",a)')trim(profile)
	    write(errmsg(3),'("NON-FATAL ERROR 205 DBG=F profile name: ",a)') trim(profinp);
	    write(errmsg(1),'("NON-FATAL ERROR 205 DBG=F a non-standard return value was reported from analyze_positions")')
	    write(errmsg(2),'("NON-FATAL ERROR 205 DBG=F the value is: ",i3)')ret;
	    call error_report(sub,205,4,errmsg); ! DBG=F qcplus - non-standard ret value from analyze positions
	endif
    endif
endif

! ==============================================================================================================
! Now that the positions have their qc flags, look for the special case of ice floats & interpolated positions

! ICE INT - variables conneted to interpolated positions/ice float
! ICE INT status is to be written to qc file only if ice lines are present OO there is an interpolation problem
! ICE INT STATUS =  0 posiiton is NOT interpolated, all ok
! ICE INT STATUS =  1 means ice lines present, position interpolated, npos=1, posloc = 1, all ok
! ICE INT STATUS =  2 means ice lines present, position interpolated, npos=3, posloc = 2, all ok
! ICE INT STATUS = -1 means ice lines present, position interpolated, npos=3, but psns 1 and/or 3 are NOT 999 lines
! ICE INT STATUS = -2 means ice lines present, position interpolated, but the number of positions is NOT 1 or 3
! ICE INT STATUS = -3 means the position is interpolated, but NO ice lines are present in the phy file

if(DBLEV==72)then; ice_lines_present = .true.;  position_interpolated = .false.; endif
if(DBLEV==73)then; ice_lines_present = .true.;  position_interpolated = .true.;  npos = 1; endif
if(DBLEV==74)then; ice_lines_present = .true.;  position_interpolated = .true.;  endif
if(DBLEV==75)then; ice_lines_present = .true.;  position_interpolated = .true.;  endif
if(DBLEV==76)then; ice_lines_present = .true.;  position_interpolated = .true.;  endif
if(DBLEV==77)then; ice_lines_present = .false.; position_interpolated = .true.;  endif
if(DBLEV > 77 .and. DBLEV < 90)then; ice_lines_present = .false.; position_interpolated = .false.; endif

if(position_interpolated)then
    if(.not. ice_lines_present)then; ! DBLEV=72
	! posloc=0; posn_cmnt="0 interp posn found, ice lines NOT found, set ice_int_status=-1";
	! from CS: Interpolation but no o ice lines is ok, just issue a waring
	if(npos == 3)then
	    posloc=2; posn_cmnt="2 interp posn found, ice lines NOT found, npos=3; set ice_int_status=-1";
	elseif(npos == 1)then
	    posloc=1; posn_cmnt="1 interp posn found, ice lines NOT found, npos=1; set ice_int_status=-1";
	endif
	write(wrnmsg,'("WARNING 139 Interpolated position present, but not ice lines were found")') ! WRN
	call warning(sub,139,wrnmsg); !WARNING 139 in  pick_profile_pos ! WRN
	ice_int_status=-1; ice_int_cmnt = ice_int_cmnts(ice_int_status); !
	if(DBGPOS)then; write(DBG,'("DBGPOSLOC: ",a)')trim(posn_cmnt); endif;
    elseif(npos == 1)then; ! position_interpolated & ice_lines_present DBLEV==73
	posloc=1; posn_cmnt="1, Set POS_QC=8, nterp posn & ice linefound, npos=1. set ice_int_status=1"; prof_POSITION_QC='8';
	ice_int_status= 1; ice_int_cmnt = ice_int_cmnts(ice_int_status);
	if(DBGPOS)then; write(DBG,'("DBGPOSLOC: ",a)')trim(posn_cmnt); endif;
    elseif(npos == 3)then
	   if((this_lat(1) >= 90.0) .and. (this_lat(3) >= 90.0))then    !check if pos 1 and 3 are 999 lines
		posloc=2; posn_cmnt="2 Set POS_QC=8, nterp posn and ice lines found, npos=3"; prof_POSITION_QC='8';
		ice_int_status= 2; ice_int_cmnt = ice_int_cmnts(ice_int_status);
		call wrtlog(sub,posn_cmnt); if(DBGPOS)then; write(DBG,'("DBG ",a)')trim(posn_cmnt); endif;
		write(wrnmsg,'("WARNING 120 ",a," lat/lon: ",f6.1,f7.1 )')trim(posn_cmnt),this_lat(posloc),this_lon(posloc) ! WRN
		call warning(sub,120,wrnmsg); ! WARNING 120 in pick_profile_pos ! WRN
	   else !  pos 1 and/or 3 are NOT 999 lines
		posloc=0; posn_cmnt="0 intice ICE npos=3 1&|3 999 posloc: "
		ice_int_status=-2; ice_int_cmnt = ice_int_cmnts(ice_int_status);
		if(DEBUG)then
		    write(errmsg(1),'("FATAL ERROR 422 this profile has an interpolated position, npos=3")'); ! DBG
		    write(errmsg(2),'("FATAL ERROR 422 however, pos 1 and/or 3 are not 999 lines")')          ! DBG
		    call error_exit(sub,422,2,errmsg); ! qcplus - intice pos=3 1&|3 not 999                   ! DBG
		else   
		    write(wrnmsg,'("WARNING 121 this profile has an interpolated position,npos=3  1&|3 not 999 lines")');  ! WRN
		    call warning(sub,121,wrnmsg); ! WARNING 121 in pick_profile_pos ! WRN
		endif   
	   endif  !end of "this_lat(1) >= 90.0)"
    else !npos not 1 and npos not 3
	posloc=0; posn_cmnt='0 ice & int ok,  npos not 1 or 3 ice_int_status set to 2';
	ice_int_status=-2; ice_int_cmnt = ice_int_cmnts(ice_int_status);
	write(posn_cmnt,'("intice, but npos not 1 or 3 posloc: ",i2)')posloc
	if(DEBUG)then
	    write(errmsg(1),'("FATAL ERROR 423 this profile has an interpolated position")')
	    write(errmsg(2),'("FATAL ERROR 423 however, the number of positions is not 1 or 3: npos= ",i2)')npos
	    call error_exit(sub,423,2,errmsg); ! qcplus - intice, npos not 1,3
	else
            write(wrnmsg,'("WARNING 122 profile has interpolated position, npos not 1 or 3: npos= ",i2)')npos ! WRN
	    call warning(sub,122,wrnmsg); ! WARNING 122 in pick_profile_pos ! WRN
	endif
	write(msg,'(a," npos=? POSIITONS MSIDENTIFIED")')trim(msg);
	if(DEBUG) write(DBG,*)'NPOS not 1/3 bad npos: ',npos,' report as error'
    endif !end of npos ==3   
    ! Note that if there is an interpolated position, it is used, so set the location/position stuff, and return
    prof_POSITION_QC = '8'; int_POSITION_QC = 8; posn_flag = 8  ! interpolated position
    if(DEBUG)then; write(DBG,'("DBG ICE & INTERPOLATION analyzed, return to the main")'); return; endif
    return; ! The interpolated position has been analyzed
endif  !end of position_interpolated     

! ======================================================================================
! Before analyzing the qc values for data/posn/land/speed, which direction do we go in the position block?
! 19Aug2015 - For DEEP_ARGO FLOATS, ust the LAST (good) position in the positon block - these have
! been transferred from the PREVIOUS PORFILE by coplete_phy, and represent the positions just before
! descent - for descending profiles that are NOT DEEP_ARGO_FLOATS, punt
if(phy_descending)then
    if(DEEP_ARGO_FLOAT)then
	posbeg = npos
	posend = 1
	posinc = -1     
    elseif(inst_type == "SOLOOF_TS10")then
	posbeg = npos
	posend = 1
	posinc = -1
    else
	write(errmsg(1),'("FATAL ERROR 405 the float is descending, but not a DEEP_ARGO_FLOAT",a)')trim(sub);
	write(errmsg(2),'("FATAL ERROR 405 the float is descending, nor is it a SOLOOF_TS10: ",a)')trim(inst_type)
	write(errmsg(3),'("FATAL ERROR 405 we cannot handle these as yet (or anymore)")');
	call error_exit(sub,405,3,errmsg); ! qcplus - "the float is descending, nor is it a SOLOOF_TS10 ???
    endif
else                         
    posbeg = 1           
    posend = npos                                                                                               
    posinc = 1
endif
! ======================================================================================           
! Before evaluating each position, see if we have an interpolated (pos_posiqc(i) = '8')
do i=posbeg,posend,posinc
    if(ipis_posnqc(i)==8)then ! A --- will be used (GPS versus Iridium does not apply)
	    ipos_posnqc = 8; position_interpolated = .true.;
	    posloc=i; write(posn_cmnt,'(a," We found an interpolated position")')trim(msg); best_position_found = .true.;
	    call wrtlog(sub,msg);

	    if(ipos_dateqc(i) /= 1 .or. ipos_posiqc(i) /= 1 .or. ipos_onland(i) /= 1 .or. ipos_speedc(i) /= 1)then
		write(errmsg(3),'("NON-FATAL ERROR 208 profile: ",a)')trim(profile)
		write(errmsg(4),'("NON-FATAL ERROR 208  in qcplus @2, phyfile: ",a)')trim(pathname_phyfile);
		write(errmsg(1),'("NON-FATAL ERROR 208  INTERPOLATED position, but one of date/posn/onland/speedc are NOT 1")')
		write(errmsg(2),'("NON-FATAL ERROR 208  date/posn/onland/speedc qc: ",4i2)')ipos_dateqc(i),ipos_posnqc(i), &
		ipos_onland(i),ipos_speedc(i)
		call error_report(sub,208,4,errmsg); ! QC+ INTERPOLATED position in phy but ...
	    endif
	    write(wrnmsg,'("WARNING 102 @1 Position ",i3," is an INTERPOLATED position")')i;  ! WRN
	    call warning(sub,102,wrnmsg); ! WARNING 102 in pick_profile_pos ! WRN
	    if(DEBUG) write(DBG,'("DBG ICE INT WRN: ",a)')(trim(wrnmsg));
	    exit;	! exit the loop, not the program
    endif    !end of "(ipis_posnqc(i)==8)"
enddo  
 
if(best_position_found)then;  ! ??? OLD ???? do we need to test this again?
    prof_POSITION_QC = '8'; int_POSITION_QC = 8; posn_flag = 8  ! interpolated position
    write(posn_cmnt,'("POSITION_QC is set to 8 from the interpolated position ")')
    if(DEBUG)then; write(msg,'("POSITION_QC @4 is set to 8 best position is 8")'); call wrtlog(sub,msg); endif
    return !If there is a non-interpolated good position, this should be set as best_position. 
endif
! ================================================================================================================
	                                 !NEW SECTION Added in November 2018
! ================================================================================================================		
! Implementing the new logic to choose the best position possible here:
posloc_iri=0;posloc_noniri=0;    
posloc=0 ; !Reset posloc if it is not an interpolated position

!==================================================================================================================
!TEST-ONLY
!find the maximum flag of all tests for each position 
!ipos_dateqc(1)=1;ipos_posiqc(1)=4;ipos_speedc(1)=4;   ipis_posnqc(1)=4; !Should we consider "ipis_posnqc(i)" along with other test results?
!ipos_dateqc(2)=1;ipos_posiqc(2)=4;ipos_speedc(2)=4;   ipis_posnqc(2)=4;
!ipos_dateqc(3)=1;ipos_posiqc(3)=1;ipos_speedc(3)=4;   ipis_posnqc(3)=1;
!==================================================================================================================
do i=posbeg,posend,posinc 
   ipos_allqc(i)=max(ipos_dateqc(i),ipos_posiqc(i),ipos_onland(i),ipos_speedc(i)); 
enddo                                            
                      
do i=posbeg,posend,posinc
    if(abs(this_lat(i)) > 90.0 .and. abs(this_lon(i)) > 180.0)cycle ! This is a 999 line
    if(badgpsfix_pos(i))then
	if(DEBUG) write(DBG,'("DBG BADGPS position ",i2," was found to have a bad gps fix.")')i
	cycle; ! Skip to the end of the do loop
    endif	
    if(ipos_posiqc(i)==1)then
          if(ipos_onland(i)==1)then
             if(ipos_speedc(i)==1)then
                if(ipos_dateqc(i)==1)then     !all are good                                                      
                   call find_poslocs(sub,i);                                              
                else ! if ipos_dateqc(ii)==1  !date is bad, all ohers are good   
                   call find_poslocs(sub,i);  
                endif
             else      !if(ipos_speedc(i)     !speed is bad
                if(ipos_dateqc(i)==1)then     !speed is bad & date is good         
                   call find_poslocs(sub,i);               
                else ! if ipos_dateqc(ii)==1  !speed is bad, date is bad
                   call find_poslocs(sub,i);                
                endif                                                                    
             endif  
          else   !ipos_onland(i)==1)...       !On Land
             if(ipos_speedc(i)==1)then        !On Land,  speed is good
                if(ipos_dateqc(i)==1)then     !On Land,  speed & date are  good  
                   call find_poslocs(sub,i);                
                else ! if ipos_dateqc(ii)==1  !On Land, speed is good, date is bad
                   call find_poslocs(sub,i);   
                endif
             else      !if(ipos_speedc(i)     !On Land, speed is bad
                if(ipos_dateqc(i)==1)then     !On Land, speed is bad & date is good    
                   call find_poslocs(sub,i);               
                else ! if ipos_dateqc(ii)==1  !On Land, speed is bad & date is bad
                   call find_poslocs(sub,i);                
                endif                                                                       
             endif         	   
          endif
    else ! (ipos_posiqc(i)==1)then,  position is off the Globe
        write(wrnmsg,'("WARNING 123 this profile has a position that is not on the Globe!")'); ! WRN
        call warning(sub,123,wrnmsg);	 ! WARNING 123 in pick_profile_pos ! WRN
    endif 
enddo   

! Now decide which one to be chosen as the best possible position, is it a GPS or an Iridium position?
if(posloc_noniri /= 0 .and. posloc_iri == 0)then  ! ! Use GPS fix, IR not available
    posloc=posloc_noniri; msg = "Use GPS fix, IR not available (Case 1)";
elseif(posloc_noniri /= 0 .and. posloc_iri /= 0)then    ! Bpth IR & GPS available
    if(ipos_allqc(posloc_noniri)==1)then
	posloc=posloc_noniri; msg = "Use GPS - Both available, GPS qc=1 (Case 3)";
    elseif(ipos_allqc(posloc_noniri)== 4 .and. ipos_allqc(posloc_iri)== 1)then
	posloc=posloc_iri; msg = "Use IR - Both available, but GPS qc=4,  IR qc=1 (Case 4)";
    else
	posloc=posloc_noniri; write(msg,'("Use GPS - Both available, but (Case 5) ??? GPS qc: ",i2," and IR qc: ",i2)') &
	    ipos_allqc(posloc_noniri),ipos_allqc(posloc_iri)
    endif
elseif(posloc_iri /= 0)then          ! Use IR, GPS not available
    posloc=posloc_iri; msg = "Use IR Fix, GPS not available (Case 2)";
endif
if(posloc == 0)then;
   posn_cmnt = '0 There is no suitable position';
elseif(posloc > 9)then;
    write(posn_cmnt,'(i2,x,a)')posloc,trim(msg);
else
    write(posn_cmnt,'(i1,x,a)')posloc,trim(msg);
endif
if(posloc /= 0)then; !Valid position found
   if(ipis_posnqc(posloc)/= 0 .and. ipos_allqc(posloc) /= ipis_posnqc(posloc))then !PI Suppiled QC is valid, but not matching with real-time POSQ : ipos_allqc(posloc)
      if(ipis_posnqc(posloc) > ipos_allqc(posloc))then  !PI supplied QC indicates bad QC compared to the real-time POSQ
        write(prof_POSITION_QC,'(i1)')ipis_posnqc(posloc);
        int_POSITION_QC=ipis_posnqc(posloc);
        write(posn_cmnt,'("PI supplied POSQ > real-time POSQ, posloc: ",i3)')posloc;
	if(DEBUG .and. found_pis_posnqc_in_phy)then; write(DBG,1001)int_POSITION_QC,pis_posnqc_phy; endif
	1001 format("DBG PI-SUPPLIED POSIITON_QC is present int he phy file: PQC: ",i2," Phyqc: ",i2);
        write(wrnmsg,'("WARNING 126 PI Suppld qc:",i3" > real-time POSQ:",i3, " @posloc =",i3, ", So using PI Suppld qc ")')& ! WRN
        ipis_posnqc(posloc), ipos_allqc(posloc),posloc
        call warning(sub,126,wrnmsg);  ! WARNING 126 in pick_profile_pos ! WRN
      else 
        write(prof_POSITION_QC,'(i1)')ipos_allqc(posloc);
        int_POSITION_QC=ipos_allqc(posloc); 
        write(posn_cmnt,'("PI supplied POSQ < real-time POSQ, posloc: ",i3)')posloc;  
        write(wrnmsg,'("WARNING 126 PI Suppld qc:",i3" < real-time POSQ:",i3, " @posloc =",i3, ", using the real-time POSQ ")')& ! WRN
        ipis_posnqc(posloc), ipos_allqc(posloc),posloc
        call warning(sub,126,wrnmsg); ! WARNING 126 in pick_profile_pos ! WRN
      endif          	   
   else
      if(ipos_allqc(posloc)==1)then;prof_POSITION_QC='1';int_POSITION_QC=1;
      elseif(ipos_allqc(posloc)==2)then;prof_POSITION_QC='2';int_POSITION_QC=2;
      elseif(ipos_allqc(posloc)==3)then;prof_POSITION_QC='3';int_POSITION_QC=3;
      elseif(ipos_allqc(posloc)==4)then;prof_POSITION_QC='4';int_POSITION_QC=4;
      else
         write(wrnmsg,'("WARNING 124 real-time POSQ, ipos_allqc=",i3," is not 1 or 4 @posloc: ",i3)')ipos_allqc(posloc),posloc; ! WRN
         call warning(sub,124,wrnmsg); ! WARNING 124 in pick_profile_pos ! WRN
      endif
      write(posn_cmnt,'("best possible position posloc: ",i3)')posloc;
   endif
else  !No Valid position found
   write(posn_cmnt,'("No valid position found.")');
   write(wrnmsg,'("WARNING 125 No valid position found, Check the possibility of an interpolated position, profile: ",a)')& ! WRN
        trim(profile);
   call warning(sub,125,wrnmsg); ! WARNING 125 in pick_profile_pos ! WRN
   !OLD write(errmsg(2),'("NON-FATAL ERROR 223 profile: ",a)')trim(profile)
   !OLD write(errmsg(3),'("NON-FATAL ERROR 223  in qcplus @2, phyfile: ",a)')trim(pathname_phyfile);
   !OLD write(errmsg(1),'("NON-FATAL ERROR 223  Could NOT find a valid position npos = ",i3)')npos
   !OLD call error_report(sub,223,3,errmsg);      
endif
if(posloc > 9)then
    write(msg,'("Position @posloc = ",i2," is chosen as the Profile Position, prof_POSITION_QC=",i3)') posloc,int_POSITION_QC;
else
    write(msg,'("Position @posloc = ",i1," is chosen as the Profile Position, prof_POSITION_QC=",i3)') posloc,int_POSITION_QC;
endif
call wrtlog(sub,msg); 
return;
end subroutine  pick_profile_pos  
! ======================================================================================            
!Newly added in November 2018
subroutine find_poslocs(caller,indx)
      
use communication                       ! module   
use data_arrays                         ! module

character(*), intent( in )     :: caller
integer, intent( in )          :: indx ! index of position array                                  
 
if(trim(trans_type) == 'IRIDIUM' .and. (sat_posn(indx) == "I" .or. sat_posn(indx) == "i"))then
  if(posloc_iri==0)then; posloc_iri=indx;                                
  elseif(ipos_allqc(posloc_iri)==4 .and. ipos_allqc(indx)==1)then;posloc_iri=indx; endif                   	
else  ! of if(trim(trans_type) == 'IRIDIUM' .and. .....                                                
  if(posloc_noniri==0)then;posloc_noniri=indx;
  elseif(ipos_allqc(posloc_noniri)==4 .and. ipos_allqc(indx)==1)then; posloc_noniri=indx; endif          
endif                                                                                                    
end subroutine find_poslocs                                                                                          
! ======================================================================================
