! ===========================================================================================================
! integer function read_phy.f90
! 
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20191126 VH     Populate variables used to select JULD, times, qc, for non-MCLines files (TIME_ASCENT_END &
!                 TIME_TRANS_START) measurement codes (MC600,MC700,MC589,MC590 & positions for ASC;
!                 MC100, MC090, MC099 for DSC), and run check_date_qc on each, to find bad gps fixes (which can
!                 affect which variables to retain. (Ex: if MC100 is good, do not populate MC090 or MC099).
! 20190904 VH     Identify bad gps fixes in the position block, later to be corrected
! 20190521 VH     If col 39 is blank, require col 40 is also blank. (ie, Look for offset 'value' in header 
!                 lines (value starts in col 40, not 41).) If not, issue a non-fatal warning, and fix the line.
!                 Eliminate errmsg(1) if only the subroutine name is present. Renumber error codes. other changes
!                 to warning and error messages.
! 20190416 VH     More DBGDIR statements.
! 20190301 JN     Handle the files with no primary core argo profile, but only secondary profile (from SIO & PMEL)
! 20190212 JN     Fix the discrepancy between units and values in the QC/NCPROF files for the floats containing 
!                 CONDUCTIVITY.
! 20181017 VH     Minor changes for defining the DIRECTION
! 20180724 VH     INTERPOLATION/ICE DETECTION: Read & store lines about Ice Detection
!                 Finalize cyc<6 warning/lognote
! 20180709 VH     Add the check for ICE DETECT lines, to coordinate with interpolated positions
! 20180628 VH     If surf_pres>20, call lognote for cyc=>6 wrn/lgn for surf_pres>20
! 20180619 VH     Convert START_OF_TRANS. & EPOCH from 19ch input into 14ch variable
!                 convert character function parse_apexsbd_config_dbar into subroutine parse_apexsbd_config_dbar(vargo)
!                 Add parsing START OF TRANSMISSION (YYYYMMDDHHMMSS) from header
!                 Read PROFILE_UPLOAD_TIME if available
! 20180417 VH     If surf_pres>20, call warning only if cyc<6 (ie, a new float). General WARNING revisions.
! 20180307 VH     Significant changes to accomodate v3.1 BR files and no-primary-profile cases
! 20180129 VH     Numerous changes for biop.
! 20180111 JN     Add checks for "no primary profile" (merged 20180220)
! 20170809 VH     Special case ICE DETECT ... MEANING may be unfilled in cycle 000 - fill with N/A
!                         issue a CRITICAL WARNING   (later elinimated)
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
! TEMP units DEGC DEG C deg... See notes in read_pqc.f90
! ===========================================================================================================

integer function read_phy(phyfile)

use communication                       ! module
use data_arrays                         ! module
use ncmeta_arrays                       ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(len=128) :: phyfile, stem
character(len=3) :: testch
integer :: status, ios, ic, nback; 
integer :: beg, end
logical :: file_exist

character(len=5)  :: mcode	! String for measurement code
character(len=11) :: clat, clon	! String for latitude, longitude
character(len=10) :: date10;	! String to read position date
character(len=8)  :: time8;	! String to read position time
character(len=3)  :: xmis       ! String to read the 3ch XMIS thingy
character(len=1)  :: csat	! String to hold the satellite
character(len=1)  :: clorqc	! characters to read the CLASS or the POSN_QC from the position block
character(len=1)  :: class      ! character to hold the CLASS
character(len=1)  :: pisqc	! character to hold the PI-Supplied QC flag
character(len=128) :: read_from	! Temporary string to hold de-double-blanked position line (to read from)
integer :: uncer, pcyc		! integers to hold the uncertainty and cycle for the soloii floats
integer :: mcinp		! The number of mc lines actually read
real :: temp_prof_pres		! Use to test the maximum_prof_pres from the phy file
real :: real_value		! Use to read int_CONFIG_MISSION_NUMBER if a decimal point is present (Duh, stupid phy file)

integer :: ll

character(len=216) :: vert_scheme_sbd;
logical :: fortraj_line
logical :: gps_nofix = .false., gps_tofix = .true.; ! Does the date need to be fixed for a bad gps fix?

integer :: doxyicol(MAXHED);
character(len=SZC) :: doxylin(MAXHED);    ! Stash the DOXY header lines (NOT including PRES/DOXY)
character(len=SZC) :: totrim
integer :: doxycol(MAXHED);               ! Which input column holds the data
integer :: doxypar(MAXHED);               ! Which parameter number goes wih which doxyline
integer :: doxyprf(MAXHED);               ! Which profile goes with which doxyline
integer :: j;
integer :: idate(6)
integer :: ninp; ! Number of input lines read
integer :: jint(6); ! Used to translate 19ch dates into 14ch
integer :: nch ! Use for offset variables
character(len=10) :: posnchar;
character(len=14), parameter :: sub = 'read_phy'

! ===========================================================================================================
! executable statements

if(DEBUG)then; write(msg,'("read_phy beg zerolines: ",l1)')zerolines; call wrtdbg(sub,msg); endif

read_phy = FAILURE;			! Assume failure

if(DBGIO) call dbg_enter(sub);

inp_open=open_old_file(INP,phyfile,"INP","pyy file");
write(msg,'("read  PHY: ",a)')trim(phyfile); call wrtstd(sub,msg);

!WRONG - CS    phy_descending = .false.;chr_DIRECTION = 'A';	! ASCDSC default - new dsc floats have CONFIG DIRECTION=2
if(DBGDIR) write(DBG,'("DBGDIR read_phy set DIR to ",a," as DEFAULT")')chr_DIRECTION

HED = .true.;  POS = .false.; DAT = .false.; ! we start n the header, duh
nhed = 0; 	! How many header records are there?
nmc = 0;	! keep track of the number of measurement code lines 
icolinp = 0;	! keep track of the inp columns
primary_prof_found=.true.; ! assume primary profile is present, set later if not.

if(DBLEV==37)then;
    write(msg,'("DBGPRESADJ read_phy  bef read HED, found_surf_pres: ",l1," surf_pres: ",f12.2)')found_surf_pres,surf_pres;
    call wrtlev(sub,msg);
endif
if(DOXY_float) nbdx = 0;
if(DBGIO) ninp = 0;

1000 format("DBGFIX read_phy NMC DATE/TIME found: ",a14," QC: ",a1," BADFIX: ",l2," def: ",a);
1001 format("DBGFIX read_phy ASC DATE/TIME found: ",a14," QC: ",a1," BADFIX: ",l2," def: ",a);
1002 format("DBGFIX read_phy ADP DATE/TIME found: ",a14," QC: ",a1," BADFIX: ",l2," def: ",a);
1003 format("DBGFIX read_phy DSC DATE/TIME found: ",a14," QC: ",a1," BADFIX: ",l2," def: ",a);
1004 format("DBGFIX read_phy DDP DATE/TIME found: ",a14," QC: ",a1," BADFIX: ",l2," def: ",a);
1005 format("DBGFIX read_phy POS DATE/TIME found: ",a14," QC: ",a1," BADFIX: ",l2," def: POS ",i1)


do while(HED)
    read(INP,'(a)',end=990,err=991,iostat=ios) cline;
    if(cline(1:12) == "============")then
	HED=.false.    ! DO NOT store this line, skip down after reading the pos header line
	read(INP,'(a)',end=990,err=991,iostat=ios) cline; call check_inp(sub,phyfile,cline);
	if(index(cline,"LATITUDE  LONGITUDE YEAR/MO/DY") ==0) go to 989; ! Do not know if this will ever come, but check it out
	go to 101       ! Skip to end of reading the header
    endif
    ! 20170807 Special case of an error in a phy file tht WHOI can't take time to fix - so, as a one-time favor
    ! we're just going to fix it ourselves - thie only occurrs in cycl 000
    if(trim(cline) == "ICE DETECT FLOAT STATUS MEANING")then
	write(cline,'("ICE DETECT FLOAT STATUS MEANING         N/A")')
	write(wrnmsg,'("WARNING 131 read_phy The ICE DETECT MEANING BLANK for phy file: ",a)')trim(profile); ! WRN
	call warning(sub,131,wrnmsg); ! WARNING 131 in read_phy ! WRN
    endif
    call check_hed(sub,phyfile,cline);
    if(cline(1:2) == "MC")then;
	nmc = nmc + 1;
	if(nmc > MAXMCL)then
	    write(errmsg(1),'("FATAL ERROR 926 nbr of mc lines exceeds MAX in read_phy (1st read)")');
	    write(errmsg(2),'("FATAL ERROR 926 number of mc lines exceeds MAXMCL")');
	    write(errmsg(3),'("FATAL ERROR 926 increase MAXMCL in data_arrays")');
	    call error_exit(sub,926,3,errmsg); ! read_phy number of mc lines exceeds MAXMcl
	endif
	mcline(nmc) = trim(cline);
    endif
    if(.not. config_lines_present .and. cline(1:6)=="CONFIG") config_lines_present = .true.;
    ! The "all equals" line delineates the end of the header/beginning of the nav block
    ! Look for the pos header line without the 'all equals' sign - it has happened in the past
    if(index(cline,"LATITUDE  LONGITUDE YEAR/MO/DY") > 0)then
	HED=.false.; ! equals line is missing between hed & nav
	write(wrnmsg,'("WARNING 132 read_phy No ============ between HED and POS in phy file: ",a)')trim(phyfile); ! WRN
	call warning(sub,132,wrnmsg); ! WARNING 132 in read_phy ! WRN
	go to 101! Skip to end of reading the header
    endif
    ! Note that if there is not value to go with the varilable, the error will be reported by check_hed
    ! However, we do still need to skip this line
    if(len(trim(cline)) < 41)then;
	write(msg,'("No value for variable: ",a)')trim(cline); call wrttrc(sub,msg);
	nhed = nhed + 1;
	if(nhed >= MAXHED)then
	   write(errmsg(1),'("FATAL ERROR 924 Attempted to read more hedlines - exceeded MAXHED (1st read)")')
	   write(errmsg(2),'("FATAL ERROR 924 Increase the size of MAXHED in data_arrays")')
	   call error_exit(sub,924,2,errmsg); ! read_phy increase the size of MAXHED
	endif
 
	hedline(nhed)=trim(cline); ! The lines are usually stored later, do it now
	go to 100 ! Skip over adalyzing this line
    endif
    ! Before parsing the input line, sii if there is an offset in the value part:
    if((cline(39:39) .eq. " ") .and. (cline(40:40) .ne. " "))then
	! If the offset is in the name of a column, if so, fix it!
	write(errmsg(1),'("NON-FATAL ERROR 301 in read_phy, a header line has a column offset")');
	write(errmsg(2),'("NON-FATAL ERROR 301 Before fix: ",a)')trim(cline);
	nch = len(trim(cline));
	cline(40:nch+1) = cline(39:nch);
	write(errmsg(3),'("NON-FATAL ERROR 301  After fix: ",a)')trim(cline);
	call error_report(sub,301,3,errmsg);
    endif
		
    read(cline,'(a40,a)')variable,value; value = trim(value);
    if(DBGIO) write(DBG,*)'   cline: ',trim(cline);
    if(trim(variable) == "NUMBER OF COLUMNS")then
	read(value,*) ncolnoc ! number of cols in the NUMBER OF COLUMNS line
	go to 100	! Skip over storing column stuff as a header line
    endif

    if(index(cline,"COLUMN    ") > 0)then
        if(trim(value) == "PRESSURE_ADJ (DBAR)")then
            ncolinp_elim = ncolinp_elim+1; ! do NOT save PRESSURE_ADJ as an old column
        elseif(trim(value) == "PSAL_ADJUSTED (PSU)")then
            ncolinp_elim = ncolinp_elim+1; ! do NOT save PSAL_ADJUSTED as an old column
        else
            icolinp = icolinp + 1; inpcols(icolinp) = trim(value)       ! inp columns
        endif
	go to 100	! Skip over storing column stuff as a header line
    endif
    if(trim(variable) == "MOST RECENT VALID SURFACE PRES (DBAR)")then ! CS says to ignore mrvsp from phy file OLD PHYS
	go to 100; ! skip storing this line
    endif
    nhed = nhed + 1;
    if(nhed >= MAXHED)then
	write(errmsg(1),'("FATAL ERROR 925 Attempted to read more hedlines - exceeded MAXHED (2dn read)")')
	write(errmsg(2),'("FATAL ERROR 925 Increase the size of MAXHED in data_arrays")')
	call error_exit(sub,925,2,errmsg); ! read_phy increase the size of MAXHED
    endif
    hedline(nhed)=trim(cline);
    if(DBGIO)then; write(msg,'("DBGIO INP PHY HED ",i6,x,a)')nhed,trim(cline); call wrtdbg(sub,msg); endif
    if(trim(variable) == "WMO RECORDER TYPE (TABLE 4770)")then
	chr_WMO_RECORDER_TYPE = trim(value)
    elseif(trim(variable) == "SURFACE PRESSURE (DBAR)")then
	read(value,*) surf_pres; found_surf_pres = .true.; ! Read in as surf_pres, not pres_offset
	if(surf_pres > 20.0)then
	    write(wrnmsg,'("WARNING 134 Surface pressure exceed 20db: ",f12.2)')surf_pres;  ! WRN
	    if(thiscyc<6)then;
		call warning(sub,134,wrnmsg); ! WARNING 134  WRN<6 surf_pres > 20 ! WRN
	    else
		call lognote(sub,134,wrnmsg); ! LOGNOTE 134 LGN<6 surf_pres > 20
	    endif
	endif
	surf_pres_comment = "SURFACE PRESSURE IN PHY FILE - NOT VALIDATED";
    elseif(trim(variable) == "SURFACE PRESSURE TRUNCATED +5 (DBAR)")then
	read(value,*) surf_pres; surf_pres = surf_pres- 5.0; found_surf_pres = .true.; ! read as surf_pres, not pres_offset
	surf_pres_comment = "SURFACE PRESSURE IN PHY FILE - NOT VALIDATED";
    elseif(trim(variable) == "PROFILES OVERLAP")then
	if(value(1:1) == 't')then
	    profiles_overlap = .true.
	elseif(value(1:1) == 'T')then
	    profiles_overlap = .true.
	endif
    elseif(trim(variable) == "TIME ASCENT END (YYYYMMDDHHMMSS)")then            ! NMC 1st JULD choice for ASC
        TIME_ASCENT_END_found = .true.;
	TIME_ASCENT_END = trim(value); 
	TIME_ASCENT_END_QC = check_cdate_qc(TIME_ASCENT_END); if(TIME_ASCENT_END_QC == 'G') badgpsfix_ascnt = .true.;
	if(TIME_ASCENT_END_QC == '1') TIME_ASCENT_END_good = .true.;
	if(DEBUG) write(DBG,*)'DATE_QC read HED TAE:     qc: ',TIME_ASCENT_END_QC,' ',TIME_ASCENT_END
    elseif(trim(variable) == "START OF TRANSMISSION")then		!19ch	! NMC 2nd JULD choice for ASC 
        TIME_TRANS_START_found = .true.;
	if(len(trim(value)) == 19)then
	    read(value,'(i4,x,i2,x,i2,x,i2,x,i2,x,i2)')jint; ! read date info from 19ch value
	    write(TIME_TRANS_START,'(i4.4,5i2.2)')jint;      ! write date into 14ch variable
	else
	    TIME_TRANS_START = trim(value);
	endif
	TIME_TRANS_START_QC = check_cdate_qc(TIME_TRANS_START); if(TIME_TRANS_START_QC == 'G') badgpsfix_strns = .true.;
	if(TIME_TRANS_START_QC == '1') TIME_TRANS_START_good = .true.;
	if(DEBUG) write(DBG,*)'DATE_QC read HED SOT1:    qc: ',TIME_TRANS_START_QC,' ',TIME_TRANS_START
    elseif(TRim(variable) == "START OF TRANSMISSION (YYYYMMDDHHMMSS)")then      ! NMC 2nd JULD choice for ASC
	TIME_TRANS_START_found = .true.; TIME_TRANS_START = trim(value);	
	TIME_TRANS_START_QC = check_cdate_qc(TIME_TRANS_START); if(TIME_TRANS_START_QC == 'G') badgpsfix_strns = .true.;
	if(TIME_TRANS_START_QC == '1') TIME_TRANS_START_good = .true.;
	if(DEBUG) write(DBG,*)'DATE_QC read HED SOT2:    qc: ',TIME_TRANS_START_QC,' ',TIME_TRANS_START
    elseif(trim(variable) == "START OF TRANSMISSION FROM EPOCH")THEN            ! NCL Used to test for bad start of trans.
        TIME_TRANS_EPOCH_found = .true.
        if(len(trim(value)) == 19)then
	    read(value,'(i4,x,i2,x,i2,x,i2,x,i2,x,i2)')jint; ! read date info from 19ch value
	    write(TIME_TRANS_EPOCH,'(i4.4,5i2.2)')jint;      ! write date into 14ch variable
        else
            TIME_TRANS_EPOCH = trim(value);
        endif
	TIME_TRANS_EPOCH_QC = check_cdate_qc(TIME_TRANS_EPOCH);  if(TIME_TRANS_EPOCH_QC == 'G') badgpsfix_epoch = .true.;
	if(TIME_TRANS_EPOCH_QC == '1') TIME_TRANS_EPOCH_good = .true.;
	if(DEBUG) write(DBG,*)'DATE_QC read HED TTE:     qc: ',TIME_TRANS_EPOCH_QC,' ',TIME_TRANS_EPOCH
    elseif(trim(variable) == "PROFILE_UPLOAD_TIME (YYYYMMDDHHMMSS)")then        ! NMC In case no other date/time available
	PROFILE_UPLOAD_TIME_found = .true.;
	PROFILE_UPLOAD_TIME = trim(value);
	PROFILE_UPLOAD_TIME_QC = check_cdate_qc(PROFILE_UPLOAD_TIME);  if(PROFILE_UPLOAD_TIME_QC == 'G') badgpsfix_upload = .true.;
	if(PROFILE_UPLOAD_TIME_QC == '1') PROFILE_UPLOAD_TIME_good = .true.;
	
! =========================================================================================================
! MC codes to look for: MC100 MC600 MC700 MC090 MC099 MC589 MC590 - Note that MC703 is only available in the pos block 
! 20191003 - notes that both MC090 & MC590 may have more than one line in a profile.  In both cases, keep the LAST GOO
! line, as it will be closest to the desired time
    elseif(variable(1:10) == "MC090 TIME")then
	! Take the LAST GOOD measurement
	TIME_TEST = trim(value); TIME_TEST_QC = check_cdate_qc(TIME_TEST);
	if(TIME_TEST_QC == '1')then
	    TIME_MC090_found = .true.; TIME_MC090 = trim(value);
	    if(TIME_MC090_QC == '1') TIME_MC090_good = .true.;
	    TIME_MC090_QC = check_cdate_qc(TIME_MC090);
	if(DEBUG) write(DBG,*)'DATE_QC read HED MC0901:  qc: ',TIME_MC090_QC,' ',TIME_MC090; ! note offset
	endif
    elseif(variable(1:10) == "MC099 TIME")then
	TIME_MC099_found = .true.; TIME_MC099 = trim(value);
	TIME_MC099_QC = check_cdate_qc(TIME_MC099); if(TIME_MC099_QC == 'G') badgpsfix_MC099 = .true.;
	if(TIME_MC099_QC == '1') TIME_MC099_good = .true.;
	if(DEBUG) write(DBG,*)'DATE_QC read HED MC099:   qc: ',TIME_MC099_QC,' ',TIME_MC099
	if(DBGFIX) write(DBG,1003)TIME_MC099,TIME_MC099_QC,badgpsfix_MC099,'TIME_MC099';
    elseif(variable(1:10) == "MC100 TIME")then
	TIME_MC100_found = .true.; TIME_MC100 = trim(value);
	TIME_MC100_QC = check_cdate_qc(TIME_MC100); if(TIME_MC100_QC == 'G') badgpsfix_MC100 = .true.;
	if(TIME_MC100_QC == '1') TIME_MC100_good = .true.;
	if(DEBUG) write(DBG,*)'DATE_QC read HED MC100:   qc: ',TIME_MC100_QC,' ',TIME_MC100
	if(DBGFIX) write(DBG,1003)TIME_MC100,TIME_MC100_QC,badgpsfix_MC100, 'TIME_MC100'
    elseif(variable(1:10) == "MC600 TIME")then
	 TIME_MC600_found = .true.; TIME_MC600 = trim(value); 
	TIME_MC600_QC = check_cdate_qc(TIME_MC600); if(TIME_MC600_QC == 'G') badgpsfix_MC600 = .true.;
	if(TIME_MC600_QC == '1') TIME_MC600_good = .true.;
	if(DEBUG) write(DBG,*)'DATE_QC read HED MC600:   qc: ',TIME_MC600_QC,' ',TIME_MC600
	if(DBGFIX) write(DBG,1001)TIME_MC600,TIME_MC600_QC,badgpsfix_MC600,'TIME_MC600';
    elseif(variable(1:10) == "MC700 TIME")then
	TIME_MC700_found = .true.; TIME_MC700 = trim(value); 
	TIME_MC700_QC = check_cdate_qc(TIME_MC700); if(TIME_MC700_QC == 'G') badgpsfix_MC700 = .true.;
	if(TIME_MC700_QC == '1') TIME_MC700_good = .true.;
	if(DEBUG) write(DBG,*)'DATE_QC read HED MC700:   qc: ',TIME_MC700_QC,' ',TIME_MC700
	if(DBGFIX) write(DBG,1001)TIME_MC700,TIME_MC700_QC,badgpsfix_MC700,'TIME_MC700';
    elseif(variable(1:10) == "MC589 TIME")then
	! NOTE THE DIFFERENCE WITH MC589 & MC590 - both may have multiple entries, and they are
	! NOT NECESSARILY in chrorological order. Test each value, and keep the latest one
	TIME_TEST = trim(value); TIME_TEST_QC = check_cdate_qc(TIME_TEST);
	if(TIME_TEST_QC == '1')then; ! Keep only good values
	    if(TIME_MC589_found)then; ! We halready ave at least one, keep the latest good one
		if(TIME_TEST > TIME_MC589)then
		    TIME_MC589 = TIME_TEST; TIME_MC589_QC = TIME_TEST_QC;
		    if(DBGFIX) write(DBG,'("DBG MC589 Swp value ",a)')TIME_MC589
		else
		    if(DBGFIX) write(DBG,'("DBG MC589 NOT value ",a)')TIME_MC589
		endif
	    else
		TIME_MC589_found = .true.; TIME_MC589 = trim(value); TIME_MC589_QC = TIME_TEST_QC;
		if(DBGFIX) write(DBG,'("DBG MC589 Set value ",a)')TIME_MC589
	    endif
	endif
    elseif(variable(1:10) == "MC590 TIME")then
	TIME_TEST = trim(value); TIME_TEST_QC = check_cdate_qc(TIME_TEST);
	if(TIME_TEST_QC == '1')then; ! Keep only good values
	    if(TIME_MC590_found)then; ! We halready ave at least one, keep the latest good one
		if(TIME_TEST > TIME_MC590)then
		    TIME_MC590 = TIME_TEST; TIME_MC590_QC = TIME_TEST_QC;
		    if(DBGFIX) write(DBG,'("DBG MC590 Swp value ",a)')TIME_MC590
		else
		    if(DBGFIX) write(DBG,'("DBG MC590 NOT value ",a)')TIME_MC590
		endif
	    else
		TIME_MC590_found = .true.; TIME_MC590 = trim(value); TIME_MC590_QC = TIME_TEST_QC;
		if(DBGFIX) write(DBG,'("DBG MC590 Set value ",a)')TIME_MC590
	    endif
	endif
! ========================================================================================================
    elseif(trim(variable) == "CONFIG PRES START CONTINUOUS PROF(DBAR)")then
	check_primary = .true.; read(value,*)pres_start_cont		! press to start hi-res
    elseif(trim(variable) == "POSITIONING SYSTEM")then
	chr_POSITIONING_SYSTEM = value(1:8);
	POSITIONING_SYSTEM_from = 'POSITIONING_SYSTEM FROM                 PHY_FILE POSITIONING_SYSTEM'
    elseif(trim(variable) == "POSITIONING_SYSTEM")then
	chr_POSITIONING_SYSTEM = value(1:8);
	POSITIONING_SYSTEM_from = 'POSITIONING_SYSTEM FROM                 PHY_FILE POSITIONING_SYSTEM'
	if(DBLEV==49)then; 
	    write(DBG,'("read_phy @2 Set chr_POSITIONING_SYSTEM=",a," (POSITIONING SYSTEM)")')trim(value); ! DBG LEV49
	    call wrtdbg(sub,msg);
	endif
    elseif(trim(variable) == "POSITION_QC")then
	  found_pis_posnqc_in_phy= .true.; read(value,*)pis_posnqc_phy
	  if(DBGPOS)then; write(DBG,'("DBG POSLOC rphy @01 pispqc: ",i3," POSITION_QC found in phy file")')pis_posnqc_phy; endif
	  write(TRC,*)'DBG POSLOC PIS pis_posnqc_phy: ',pis_posnqc_phy;
    elseif(trim(variable) == "CONFIG DIRECTION")then
	if(trim(value) == "1")then
	    chr_DIRECTION = "A";
!WRONG - CS	    phy_descending = .false.;  chr_DIRECTION = "A";
            if(DBGDIR) write(DBG,'("DBGDIR read_phy set DIR to ",a," from CONFIG DIRECTION")')chr_DIRECTION
	    direction_configdir = "A"
	elseif(trim(value) == "2")then
	    chr_DIRECTION = "D";
!WRONG - CS	    phy_descending = .true.;  chr_DIRECTION = "D";
            if(DBGDIR) write(DBG,'("DBGDIR read_phy set DIR to ",a," from CONFIG DIRECTION")')chr_DIRECTION
	    direction_configdir = "D";
	else
	    if(thiscyc == 0)then; ! Example: SOLO2IR_TS36 - cyc=000. CONFIG DIRECTION fill, but ASSUME the prof is asc.
		chr_DIRECTION = "A";
!WRONG - CS		phy_descending = .false.;  chr_DIRECTION = "A";
                if(DBGDIR) write(DBG,'("DBGDIR read_phy set DIR to ",a," from CONFIG DIRECTION CYC000")')chr_DIRECTION
		direction_configdir = "N"
	    else
		chr_DIRECTION = "N";
		direction_configdir = "N"
	    endif
	endif
        if(DBGDIR) write(DBG,'("DBGDIR read_phy set DIR to ",a," from CONFIG DIRECTION")')chr_DIRECTION
	if(DEBUG)then; write(msg,'("read_phy sets DIRECTION to ",a," @1")')chr_DIRECTION; call wrtdbg(sub,msg); endif
    elseif(trim(variable) == "DIRECTION OF PROFILES")then
	chr_DIRECTION = value(1:1); ! In spite of the number of profiles, ALL ARE THE SAME DIRECTION!!!!
	direction_DOPs = value(1:1)
	if(DEBUG)then; write(msg,'("read_phy sets DIRECTION to ",a," @2")')chr_DIRECTION; call wrtdbg(sub,msg); endif
        if(DBGDIR) write(DBG,'("DBGDIR read_phy set DIR to ",a," from DIRECTION OF PROFILES")')chr_DIRECTION
    elseif(trim(variable) == "CONFIG FLOAT MISSION")then
	config_float_mission_present = .true.
	read(value,*,err=997)int_CONFIG_MISSION_NUMBER
	go to 998
	997 continue ! Come here for an error in reading the int
	read(value,*)real_value;
	int_CONFIG_MISSION_NUMBER = int(real_value)
	write(errmsg(1),'("NON-FATAL ERROR 302 in in read_phy - error reading int_CONFIG_MISSION_NUMBER")');
	write(errmsg(2),'("NON-FATAL ERROR 302 CONFIG FLOAT MISSION value: ",a)')trim(value);
	call error_report(sub,302,2,errmsg); ! read_phy error reading int_CONFIG_MISSION_NUMBER
	998 continue ! Come here if the int was read ok
	if(int_CONFIG_MISSION_NUMBER > -900)then			! John uses -999 as fill value for CMN
	else
	    int_CONFIG_MISSION_NUMBER = FILLINT

	endif
    elseif(trim(variable) == "CONFIG PROFILE PRESSURE (DBAR)")then
	read(value,*)temp_prof_pres		! Make sure it is not fill value
	if(temp_prof_pres > -100)then
	    maximum_prof_pres_from = "PHY_FILE"	! This overrides the value(s) from the meta file
	    read(value,*)maximum_prof_pres		! This will now be used in qc19
	    write(msg,'("DEEPEST_PRES In read_phy  set maximum_prof_pres: ",f12.2)')maximum_prof_pres;
	else
	    write(msg,'("DEEPEST PRES In read_phy will NOT! reset from phy file, it is fill value: ",f12.2)')temp_prof_pres;
	endif
	call wrtlog(sub,msg); 
    elseif(trim(variable) == "1. VERTICAL SAMPLING SCHEME")then
	sampling_found = .true.; if(sampling_entries < 1) sampling_entries = sampling_entries + 1;
	VERT_SAMPLING_SCHEME_FROM = "PHY_FILE";
	chr_VERT_SAMPLING_SCHEME(1) = trim(value);
	if(DEBUG)then; write(msg,'("VSS read_phy  vss1: ",a)')trim(value); call wrtdbg(sub,msg); endif
    elseif(trim(variable) == "2. VERTICAL SAMPLING SCHEME")then
	sampling_found = .true.; if(sampling_entries < 2) sampling_entries = sampling_entries + 1;
	VERT_SAMPLING_SCHEME_FROM = "PHY_FILE";
	chr_VERT_SAMPLING_SCHEME(2) = trim(value);
	if(DEBUG)then; write(msg,'("VSS read_phy  vss2: ",a)')trim(value); call wrtdbg(sub,msg); endif
    elseif(trim(variable) == "3. VERTICAL SAMPLING SCHEME")then
	sampling_found = .true.; if(sampling_entries < 3) sampling_entries = sampling_entries + 1;
	VERT_SAMPLING_SCHEME_FROM = "PHY_FILE";
	chr_VERT_SAMPLING_SCHEME(3) = trim(value);
    elseif(trim(variable) == "SBE CONFIG (DBAR)")then
	if(DEBUG)then; write(msg,'("VSS read_phy  vss3: ",a)')trim(value); call wrtdbg(sub,msg); endif
	sampling_found = .true.; if(sampling_entries < 1) sampling_entries = sampling_entries + 1;
	VERT_SAMPLING_SCHEME_FROM = "PHY_FILE";
        call parse_apexsbd_config_dbar(value,vert_scheme_sbd);
        chr_VERT_SAMPLING_SCHEME(1) = vert_scheme_sbd;
    elseif(index(variable,"ICE DETECT") > 0)then;
	! it seems that the value for any ice variable may be suspect, just look to see if ICE lines are present
	ice_lines_present = .true.;
    elseif(index(variable,"COLUMN                            ") > 0)then
	if(trim(value) == "PRESSURE_ADJ (DBAR)")then
	    ncolinp_elim = ncolinp_elim+1; ! do NOT save PRESSURE_ADJ as an old column
	elseif(trim(value) == "PSAL_ADJUSTED (PSU)")then
	    ncolinp_elim = ncolinp_elim+1; ! do NOT save PSAL_ADJUSTED as an old column
	else
	    icolinp = icolinp + 1; inpcols(icolinp) = trim(value)	! inp columns
	endif
    endif
    100 continue	! Skip over storing column stuff as a header line
enddo	! while(HED)

101 continue	! Skip to end of reading the header
if(DBGIO)then; write(msg,'("DBGIO INP HED end of HED with nhed: ",i5)')nhed;                call wrtdbg(sub,msg); endif
if(DBLEV==37)then;
    write(msg,'("DBGPRESADJ read_phy  aft read HED, found_surf_pres: ",l1," surf_pres: ",f12.2)')found_surf_pres,surf_pres;
    call wrtlev(sub,msg);
endif
if(DBLEV== 2)then; write(msg,'("LEV02 DATE Begining of pos block,   cline: ",a)')trim(cline);   call wrtdbg(sub,msg); endif
if(DBLEV== 3)then; write(msg,'("LEV03 POS Begining of pos block,   cline: ",a)')trim(cline);    call wrtdbg(sub,msg); endif

! Check out the VERTICAL SAMPLING SCHEME
if(.not. sampling_found)then
    write(wrnmsg,'("WARNING 135 read_phy The VERTICAL SAMPLING SCHEME is NOT set in either META or PHY files")'); ! WRN
    call warning(sub,135,wrnmsg); ! WARNING 135 in read_phy ! WRN
else
    if(index(chr_VERT_SAMPLING_SCHEME(1),'Primary sampling') <=0 )primary_prof_found=.false.;
    if(index(chr_VERT_SAMPLING_SCHEME(1),'Primary sampling: averaged [No Argo Core Profile]')>0)primary_prof_found=.false.;
    ! if(.not. primary_prof_found)then; nprf_start=2; else; nprf_start=1; endif
endif

! Parse the position label line to see what we have
! First, stash the line as we have it now (modify later)
!  CLASS lines, posqc1:  LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS
! POS-QC lines, posqc2:  LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT POSITION-QC
! POS_QC lines, posqc3:  LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT POSITION_QC
! MCODE  lines, posqc4:  LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT POSN-QC UNCER CYC
!                >123451234567890
! MCODE  lines,  >       LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT POSN-QC UNCER CYC 
! MCODE   data,  >MC703 -33.85068 -126.28966 2014/04/08 16:35:00   000   3    4        1  97   
if(index(cline,"CLASS") > 0)then
    if(DBLEV== 2)then; write(msg,'("LEV02 POS I The position block contains CLASS")'); call wrtdbg(sub,msg); endif
    this_class_present = .true.;
    poslabs = "   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS RTPQC RTTQC RTLQC RTSQC";
elseif(index(cline,"POSITION") > 0)then
    if(DBLEV== 2)then; write(msg,'("LEV02 POS I The position block contains POSITION")'); call wrtdbg(sub,msg); endif
    this_posqc_present = .true.;
    poslabs = "   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT POSQC RTPQC RTTQC RTLQC RTSQC";
elseif(index(cline,"POSN-QC UNCER CYC") > 0)then
    if(DBLEV== 2)then; write(msg,'("LEV02 POS I The position block contains MCODE")'); call wrtdbg(sub,msg); endif
    this_mcode_present = .true.;
    poslabs = "   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT POSQC RTPQC RTTQC RTLQC RTSQC  UNCER CYC     MC";
else
    write(errmsg(1),'("FATAL ERROR 911 position label line cannot be decoded")');
    write(errmsg(2),'("FATAL ERROR 911 trim(phyfile)")');
    write(errmsg(3),'("FATAL ERROR 911 cline: ",a," ")')trim(cline);
    call error_exit(sub,911,3,errmsg); ! read_phy position label line cannot be decoded
endif
! What type of position block do we have?
if(DBLEV== 2)then								! LEV02 imp date, imp pos, land, juldqc
    write(msg,*)'LEV02 POS Position block status, this_class_present: ',this_class_present; call wrtdbg(sub,msg);
    write(msg,*)'LEV02 Position block status, this_posqc_present: ',this_posqc_present; call wrtdbg(sub,msg);
    write(msg,*)'LEV02 Position block status, this_mcode_present: ',this_mcode_present; call wrtdbg(sub,msg);
    write(msg,*)'LEV02 Position label line: ',trim(poslabs); call wrtdbg(sub,msg);
endif
ncolinp = icolinp; 
! End of header found, either ============ or LAT/LON/DATE line
if(DEBUG)then
    write(msg,'("read_phy HED2POS ",i3," = Number of input cols from NUMBER OF COLUMNS line")')ncolnoc; call wrtdbg(sub,msg);
    if(ncolinp_elim > 0)then
	write(msg,'("read_phy HED2POS ",i3," = Number of input cols to ELIMINATE from NUMBER OF COLUMNS line")')ncolinp_elim;
	call wrtdbg(sub,msg); 
    endif
    write(msg,'("read_phy HED2POS ",i3," = Number of input cols from actual number read")')icolinp; call wrtdbg(sub,msg);
endif
if((ncolnoc-ncolinp_elim) /= icolinp)then 
    write(errmsg(1),'("FATAL ERROR 912 COLUMN UMBER MISMATCH #1")');
    write(errmsg(2),'("FATAL ERROR 912 MISMATCH: ",i3," = number of input cols differs from NUMBER OF COLUMNS line: ")')ncolnoc
    write(errmsg(3),'("FATAL ERROR 912 MISMATCH: ",i3," = number of input cols to eliminate from the NOC line: ")')ncolinp_elim
    write(errmsg(4),'("FATAL ERROR 912 MISMATCH: ",i3," = number of input cols from lines of input columns: ")')icolinp
    call error_exit(sub,912,4,errmsg); ! read_phy number of input cols differs from NUMBER OF COLUMNS line
endif
if(DEBUG)then; write(msg,'("Endo of header detected, read position block")'); call wrtdbg(sub,msg); endif
HED = .false.; POS = .true.;  DAT = .false.  ! after the header, read the position
!-------------------------------------------------------
npos=0;  !npos=number of position lines (EXCLUDES nav/dat separator, 999 lines)
npos2=0; !npos2=number of position lines without the +99.999 +999.999 (EXcoludes nav/dat separator)
index_999_1 = 0;index_999_2 = 0;count_999=0;
!======================================================================================================================
! Position blocks for descending profiles have lines that start with "FORTRAJ" ignore these lines for the profile (ie.,
! don't add to the position array) but add them to the pos line array, and pass them through to the qc file (as is)
!
! Typical position block (this is for 5476_006003_054D.phy as of 02Jun2016:
!
!        LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT POSN-QC UNCER CYC
! FORTRAJ MC703 -36.07817 -177.11647 2014/06/19 03:17:00   000   6    1        9  54
! MC703 -36.07703 -177.11900 2014/06/19 02:13:00   000   7    1       11  52
! MC703 -36.067   -177.111   2014/06/19 02:14:03   000   I    1     5000  52
! FORTRAJ MC703 -36.081   -177.155   2014/06/19 03:18:11   000   I    1     4556  54
! 1234567890123456789012345678901234567890123456789012345678901234567890123456789012
if(DBGFIX)then
    if(TIME_ASCENT_END_QC .ne. "9") write(DBG,1000)TIME_ASCENT_END, TIME_ASCENT_END_QC,badgpsfix_ascnt,'TIME_ASCENT_END';	! DBG	
    if(TIME_TRANS_START_QC .ne. "9") write(DBG,1000)TIME_TRANS_START,TIME_TRANS_START_QC,badgpsfix_strns,'TIME_TRANS_START';	! DBG
    if(TIME_TRANS_EPOCH_QC .ne. "9") write(DBG,1000)TIME_TRANS_EPOCH,TIME_TRANS_EPOCH_QC,badgpsfix_epoch,'TIME_TRANS_EPOCH';	! DBG
    if(PROFILE_UPLOAD_TIME_QC .ne. "9") write(DBG,1000)PROFILE_UPLOAD_TIME,PROFILE_UPLOAD_TIME_QC,badgpsfix_upload, &		! DBG
	'PROFILE_UPLOAD_TIME';
    if(TIME_MC100_QC .ne. "9") write(DBG,1003)TIME_MC100,TIME_MC100_QC,badgpsfix_MC100,'TIME_MC100';	! DBG
    if(TIME_MC600_QC .ne. "9") write(DBG,1001)TIME_MC600,TIME_MC600_QC,badgpsfix_MC600,'TIME_MC600';	! DBG
    if(TIME_MC700_QC .ne. "9") write(DBG,1001)TIME_MC700,TIME_MC700_QC,badgpsfix_MC700,'TIME_MC700';	! DBG
    if(TIME_MC090_QC .ne. "9") write(DBG,1003)TIME_MC090,TIME_MC090_QC,badgpsfix_MC090,'TIME_MC090';	! DBG
    if(TIME_MC099_QC .ne. "9") write(DBG,1003)TIME_MC099,TIME_MC099_QC,badgpsfix_MC099,'TIME_MC099';	! DBG
     if(TIME_MC589_QC .ne. "9") write(DBG,1001)TIME_MC589,TIME_MC589_QC,badgpsfix_MC589,'TIME_MC589';	! DBG
    if(TIME_MC590_QC .ne. "9") write(DBG,1001)TIME_MC590,TIME_MC590_QC,badgpsfix_MC590,'TIME_MC590';	! DBG
endif

if(nmc > 0)then; hasMC = .true.; endif
if(hasMC .and. phy_descending)then; TIME_MC703__needed = .false.; else; TIME_MC703__needed = .true.; endif

npos = 0; npos_traj = 0;
do while(POS)
    read(INP,'(a)',end=992,err=993,iostat=ios) cline; call check_inp(sub,phyfile,cline);
    if(DBGIO)then; ninp=ninp+1; write(msg,'("INP INP ",i6)')ninp; call dbgmsg(sub,msg); endif
    if(cline(1:12) == "============")then
	if(npos == 0)then
	    write(errmsg(1),'("FATAL ERROR 917 in reading position block in rea_phy")');
	    write(errmsg(2),'("FATAL ERROR 917 in read_phy, posn block no position, but separators and label line")');
	    write(errmsg(3),'("FATAL ERROR 917 ==========================================================================")');
	    write(errmsg(4),'("FATAL ERROR 917        LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT POSN-QC UNCER CYC")');
	    write(errmsg(5),'("FATAL ERROR 917 ==========================================================================")');
	    call error_exit(sub,917,5,errmsg); ! read_phy reading position block
	endif
	if(DBLEV== 2)then;write(msg,'("LEV02 POS found end of pos lines",a)')trim(cline);call wrtdbg(sub,msg);endif
	POS=.false.;
    else
	if(index(cline,"FORTRAJ") > 0)then
	    if(cline(1:10) /= "FORTRAJ MC")then
		write(errmsg(1),'("FATAL ERROR 920 in read_phy for fle: ",a)')trim(phyfile);
		write(errmsg(2),'("FATAL ERROR 920 Improperly formatted FORTRAJ line")');
		write(errmsg(3),'("FATAL ERROR 920 line: ",a)')trim(cline);;
		call error_exit(sub,920,3,errmsg); ! read_phy improperly formatted FORTRAJ line
	    endif
	    npos_traj = npos_traj + 1;
	    posline_traj(npos_traj) = cline(14:71);
	    mcode = cline(9:13);
	    ll = len(trim(cline));
	    write(posadnl_traj(npos_traj),'(a,2x,a," FORTRAJ")')cline(72:ll),mcode

	    pos_posnqc_traj(npos_traj) = "0"; pos_dateqc_traj(npos_traj) = "0";
	    pos_onland_traj(npos_traj) = "0"; pos_speedc_traj(npos_traj) = "0";
	    
	    go to 10; ! SKIP_FORTRAJ line
	endif
	if(cline(1:2) == "MC")then;
	    nmc = nmc + 1; 
	    if(nmc > MAXMCL)then
		write(errmsg(1),'("FATAL ERROR 927 nbr of mc codes exceeds MAX in read_phy (2nd read)")');
		write(errmsg(2),'("FATAL ERROR 927 number of mc code lines exceeds MAXMCL")');
		write(errmsg(3),'("FATAL ERROR 927 increase MAXMCL in data_arrays")');
		call error_exit(sub,927,3,errmsg); ! read_phy number of mc code lines exceeds MAXMCL
	    endif
	 mcline(nmc) = trim(cline);
	endif
        npos = npos + 1;
	if(TIME_MC703__needed)then
	    if(index(cline,'I') == 0)then; ! Use only non-iridium (GPS) fixes
		if(cline(1:5) == 'MC703')then;
		    read(cline,'(27x,i4,5(x,i2))')idate;
		else
		    read(cline,'(23x,i4,5(x,i2))')idate; 
		endif
		write(TIME_TEST,'(i4.4,5i2.2)')idate
		TIME_TEST_QC = check_cdate_qc(TIME_TEST);
		if(.not. TIME_MC703_1_found)then
		    TIME_MC703_1 = TIME_TEST; TIME_MC703_1_QC = TIME_TEST_QC;
		    TIME_MC703_1_found = .true.; if(TIME_MC703_1_QC == '1') TIME_MC703_1_good = .true.;
		elseif(.not. TIME_MC703_2_found)then
		    TIME_MC703_2 = TIME_TEST; TIME_MC703_2_QC = TIME_TEST_QC;
		    TIME_MC703_2_found = .true.; ; if(TIME_MC703_2_QC == '1') TIME_MC703_2_good = .true.;
		    TIME_MC703__needed = .false.; ! We have found the first two GPS fixes we need
		endif
	    else
	    endif
	endif; ! End pf TIME_MC703__needed
	pos_posnqc(npos) = '0';        ! initialize 'full' position qc (pos_posiqc ND pis_posnqc
	pis_posnqc(npos) = '0';        ! initialize PI-supplied qc
	pos_posiqc(npos) = '0';        ! initialize impossible position
	pos_dateqc(npos) = '0';        ! initialize date qc
	pos_onland(npos) = '0';        ! initialize position on land
	pos_speedc(npos) = '0';        ! initialize speed check

	if(DBGIO)then; write(msg,'("DBGIO INP PHY POS ",i6,x,a)')npos,trim(cline); call wrtdbg(sub,msg); endif
	if(this_class_present)then
	    write(posline(npos),'(a)')trim(cline);			! Take the whole line for CLASS posn blocks
	elseif(this_posqc_present)then
	    write(posline(npos),'(a)')trim(cline);			! Take the whole line for PI-Supplied posn_qc blocks
	else
	    read(cline,'(a5)')mcode					! Read the mcode off the front of the cline
	    write(posline(npos),'(x,a)')cline(6:62)			! Save the lat/lon/date,etc, through the pos-qc
	    write(posadnl(npos),'(a,2x,a)')cline(63:len(trim(cline))),mcode
	endif
	! get the actual data out - first, squeeze the posnline, replace : & / with a space
	call squeezerep(posline(npos),read_from);
	! now read free format
	read(read_from,*)this_lat(npos),this_lon(npos),this_year(npos),this_mon(npos),this_day(npos),this_hh(npos), &
		this_mm(npos),this_sc(npos),xmis,csat,clorqc
	write(this_date_time(npos),'(i4.4,i2.2,i2.2,i2.2,i2.2,i2.2)') this_year(npos),this_mon(npos),this_day(npos), &
		this_hh(npos),this_mm(npos),this_sc(npos)
	TIME_TEST_QC = check_cdate_qc(this_date_time(npos)); this_date_time_qc(npos) = TIME_TEST_QC;
	if(DEBUG)then;
	    write(chr7,'("pos: ",i2)')npos
	     write(DBG,*)'DATE_QC read POS, ',chr7,' qc: ',this_date_time_qc(npos),' ',this_date_time(npos);  ! DBG
	endif
	if(TIME_TEST_QC == 'G')then
	    badgpsfix_pos_found = .true.; badgpsfix_pos(npos) = .true.;
	    if(DBGFIX)then;
		if(npos > 9)then; write(posnchar,'("POSITION ",i2)')npos; else; write(posnchar,'("POSITION ",i1)')npos; endif
		write(DBG,1003)this_date_time(npos),TIME_TEST_QC,badgpsfix_pos(npos),posnchar ! DBG
	    endif
	endif
	if(csat=="i" .or. csat=="I")then; sat_posn(npos) = 'I'; else; sat_posn(npos) = csat; endif

	read(read_from,*)dble_lat(npos),dble_lon(npos);
        if(this_class_present)then
	    ! Look for teh dreaded minus sign before the location class
	    ! wh "should" not get this, but some really old phy files may have it
	    ! Scream then exit with a FATAL ERROR
	    if(DBGIO)then; write(msg,*)'DBGIO TEST for minus class npos: ',npos,' clorqc: ',clorqc; call wrtdbg(sub,msg); endif
	    if(clorqc == "-")then
		write(errmsg(1),'("FATAL ERROR 915 in read_phy")');
		write(errmsg(2),'("FATAL ERROR 915 a minus signe was detected attached to the LOCATION CLASS column")');
		write(errmsg(3),'("FATAL ERROR 915 This is a VERY OLD PHY FILE, and the CLASS needs correction")');
		call error_exit(sub,913,3,errmsg); ! read_phy minus signe attached to LOCATION CLASS col
	    endif
	    pis_posnqc(npos) = '0';		! No PI-supplied posn_qc is present
        else
	    pis_posnqc(npos) = clorqc;         ! Here it is the PI-supplied posn_qc (both posqc * mcode)
	    if(DEBUG)then; write(msg,*)'read_phy found pi-pqc ipos: ',npos,' qc: ',pis_posnqc(npos); call wrtdbg(sub,msg); endif
        endif
	! evaluate the lat/on and date for fill value
	if(this_lat(npos) > 90.0 .or. this_lon(npos) > 180.0)then;
	    if(DBLEV== 5)then
		write(msg,'("LEV05 lat>90 &| lon>180, assume fill value: ",2f10.2)')this_lat(npos),this_lon(npos)
		call wrtdbg(sub,msg);
	    endif
	    pos_posnqc(npos) = '9';        ! position qc - position MISSING
	endif

        if(this_lat(npos) < 98.0)then           ! for lat, fill is +99.999
	    npos2 = npos2 + 1;                         ! npos2 is number of positins EXCLUDING 999 lines
	    this_lat2(npos2) = this_lat(npos)
	    this_lon2(npos2) = this_lon(npos)
	    this_year2(npos2) = this_year(npos)
	    this_mon2(npos2) = this_mon(npos)
	    this_day2(npos2) = this_day(npos)
	    this_hh2(npos2) = this_hh(npos)
	    this_mm2(npos2) = this_mm(npos)
	    this_sc2(npos2) = this_sc(npos)
	endif
    endif
    10 continue; ! SKIP_FORTRAJ line
enddo; ! End of (POS)
if(badgpsfix_found)then
    if(badgpsfix_pos_found)then
	write(msg,'("BADPOS There are badgps dates in the position block")'); call wrtlog(sub,msg);
	   do ipos=1,npos
	    write(msg,'("BADGPS ipos: ",i3," fix? ",l2," date/time: ",a)')ipos,badgpsfix_pos(ipos),this_date_time(ipos)
	    call wrtlog(sub,msg);
	enddo
    endif
endif
!===========================================================================================================
! NOTE THE OFFSET BEG HERE
npres=0; ntemp=0; ncndc=0; npsal=0; ndoxy=0;
do icol=1,ncolinp
    value = trim(inpcols(icol));
    if(index(value,"(dbar)") > 0)then
	write(errmsg(1),'("NON-FATAL ERROR 303 Detected no caps for units of a PRES column: ",a)')trim(value);
	call error_report(sub,303,1,errmsg); ! read_phy no caps for pres units
	call set_uppercase(value,value); call set_uppercase(cline,cline);
	inpcols(icol) = trim(value);
    elseif(index(value,"(deg") > 0)then
	write(errmsg(1),'("NON-FATAL ERROR 304 Detected no caps for units ofa TEMP column: ",a)')trim(value);
	call error_report(sub,304,1,errmsg); ! read_phy no caps for TEMP units
	call set_uppercase(value,value); call set_uppercase(cline,cline);
	inpcols(icol) = trim(value);
    elseif(index(value,"(psu)") > 0)then
	write(errmsg(1),'("NON-FATAL ERROR 305 Detected no caps for units ofa PSAL column: ",a)')trim(value);
	call error_report(sub,305,1,errmsg); ! read_phy no caps for PSAL units
	call set_uppercase(value,value); call set_uppercase(cline,cline);
	inpcols(icol) = trim(value);
    endif
    if(value == "PRESSURE (DBAR)")then
	hasPRES(1) = .true.; npres = 1; inprescol(1) = icol; ! Store pres info
    elseif(value == "PRESSURE2 (DBAR)")then
	hasPRES(2) = .true.; npres = npres + 1; inprescol(2) = icol; ! Store pres2 info
    elseif(value == "PRESSURE3 (DBAR)")then
	hasPRES(3) = .true.; npres = npres + 1; inprescol(3) = icol; ! Store pres3 info
    elseif(value == "PRESSURE4 (DBAR)")then
	hasPRES(4) = .true.; npres = npres + 1; inprescol(4) = icol; ! Store pres4 info

    elseif(value(1:19) == "TEMPERATURE-90 (DEG" .or. value(1:19) == "TEMPERATURE-90 (deg") then
	hasTEMP(1) = .true.; ntemp = ntemp + 1; intempcol(1) = icol; ! Store temp info
	inpcols(icol) = "TEMPERATURE-90 (DEG C)"
    elseif(value(1:20) == "TEMPERATURE2-90 (DEG" .or. value(1:20) == "TEMPERATURE2-90 (deg") then
	hasTEMP(2) = .true.; ntemp = ntemp + 1; intempcol(2) = icol; ! Store temp2 info
	inpcols(icol) = "TEMPERATURE2-90 (DEG C)"
    elseif(value(1:20) == "TEMPERATURE3-90 (DEG" .or. value(1:20) == "TEMPERATURE3-90 (deg") then
	hasTEMP(3) = .true.; ntemp = ntemp + 1; intempcol(3) = icol; ! Store temp3 info
	inpcols(icol) = "TEMPERATURE3-90 (DEG C)"
    elseif(value(1:20) == "TEMPERATURE4-90 (DEG" .or. value(1:20) == "TEMPERATURE4-90 (deg") then
	hasTEMP(4) = .true.; ntemp = ntemp + 1; intempcol(4) = icol; ! Store temp4 info
	inpcols(icol) = "TEMPERATURE4-90 (DEG C)"

    elseif(value == "SALINITY (PSU)")then
	hasPSAL(0) = .true.; hasPSAL(1) = .true.; npsal = npsal + 1; inpsalcol(1) = icol; ! Store psal info
    elseif(value == "SALINITY2 (PSU)")then
	hasPSAL(0) = .true.; hasPSAL(2) = .true.; npsal = npsal + 1; inpsalcol(2) = icol; ! Store psal2 info
    elseif(value == "SALINITY 2 (PSU)")then
	hasPSAL(0) = .true.; hasPSAL(2) = .true.; npsal = npsal + 1; inpsalcol(2) = icol; ! Store psal2 info
	inpcols(icol) = "SALINITY2 (PSU)"
    elseif(value == "SALINITY3 (PSU)")then
        hasPSAL(0) = .true.; hasPSAL(3) = .true.; npsal = npsal + 1; inpsalcol(3) = icol; ! Store psal3 info
    elseif(value == "SALINITY 3 (PSU)")then
        hasPSAL(0) = .true.; hasPSAL(3) = .true.; npsal = npsal + 1; inpsalcol(3) = icol; ! Store psal3 info
        inpcols(icol) = "SALINITY3 (PSU)"
    elseif(value == "SALINITY4 (PSU)")then
	hasPSAL(0) = .true.; hasPSAL(4) = .true.; npsal = npsal + 1; inpsalcol(4) = icol; ! Store psal4 info

    elseif(value == "CONDUCTIVITY")then
        hasCNDC(1) = .true.; ncndc = ncndc + 1; incndccol(1) = icol; cnvrt_cndc(1) = .true.; ! Store cndc1 info
	write(errmsg(1),'("NON-FATAL ERROR 306 Detected no unit for CONDUCTIVITY column: ",a)')trim(value);
	call error_report(sub,306,1,errmsg); ! read_phy no unit for CONDUCTIVITY column
    elseif(value == "CONDUCTIVITY (MS/CM)")then
	hasCNDC(1) = .true.; ncndc = ncndc + 1; incndccol(1) = icol; cnvrt_cndc(1) = .true.; ! Store cndc1 info
    elseif(value == "CONDUCTIVITY2 (MS/CM)")then
	hasCNDC(2) = .true.; ncndc = ncndc + 1; incndccol(2) = icol; cnvrt_cndc(2) = .true.; ! Store cndc2 info
    elseif(value == "CONDUCTIVITY3 (MS/CM)")then
	hasCNDC(3) = .true.; ncndc = ncndc + 1; incndccol(3) = icol; cnvrt_cndc(3) = .true.; ! Store cndc3 info
    elseif(value == "CONDUCTIVITY4 (MS/CM)")then
	hasCNDC(4) = .true.; ncndc = ncndc + 1; incndccol(4) = icol; cnvrt_cndc(4) = .true.; ! Store cndc4 info
    elseif(value == "CONDUCTIVITY (MHOS/M)")then
	hasCNDC(1) = .true.; ncndc = ncndc + 1; incndccol(1) = icol; ! Store cndc1 info
    elseif(value == "CONDUCTIVITY2 (MHOS/M)")then
	hasCNDC(2) = .true.; ncndc = ncndc + 1; incndccol(2) = icol; ! Store cndc2 info
    elseif(value == "CONDUCTIVITY3 (MHOS/M)")then
	hasCNDC(3) = .true.; ncndc = ncndc + 1; incndccol(3) = icol; ! Store cndc3 info
    elseif(value == "CONDUCTIVITY4 (MHOS/M)")then
	hasCNDC(4) = .true.; ncndc = ncndc + 1; incndccol(4) = icol; ! Store cndc4 info


    elseif(value == "INTERP_STATUS_T2")then
	interp_status = .true.; interp_t2_col = icol;
    elseif(value == "INTERP_STATUS_S2")then
	interp_status = .true.; interp_s2_col = icol;
    elseif(value == "INTERP_STATUS_T3")then
	interp_status = .true.; interp_t3_col = icol;
    elseif(value == "INTERP_STATUS_S3")then
	interp_status = .true.; interp_s3_col = icol;
    elseif(value == "DISSOLVED OXYGEN (MICROMOL/KG)")then
	hasDOXY(1) = .true.; ndoxy = ndoxy + 1; indoxycol(1) = icol; ! Store doxy1 info
    elseif(value == "DISSOLVED OXYGEN2 (MICROMOL/KG)")then
	hasDOXY(2) = .true.; ndoxy = ndoxy + 1; indoxycol(2) = icol; ! Store doxy2 info
    elseif(value == "DISSOLVED OXYGEN2A (MICROMOL/KG)")then
	hasDOXYA = .true.; hasDOXY(3) = .true.; ndoxy = ndoxy + 1; indoxycol(3) = icol; ! Store doxy2a info
    elseif(value == "DISSOLVED OXYGEN2B (MICROMOL/KG)")then
	hasDOXYB = .true.; hasDOXY(4) = .true.; ndoxy = ndoxy + 1; indoxycol(4) = icol; ! Store doxy2b info
    elseif(value == "DISSOLVED OXYGEN2C (MICROMOL/KG)")then
	hasDOXYC = .true.; hasDOXY(5) = .true.; ndoxy = ndoxy + 1; indoxycol(5) = icol; ! Store doxy2c info
    elseif(value == "CHECKSUM FLAG")then
	hasCKSM(1) = .true.; ncksmcol(1) = icol						! Store checksum1 info
	if(hasDATA(p2r))then
	    cksm1_in_prof1 = .false.; cksm1_in_prof2 = .true.;
	else
	    cksm1_in_prof1 = .true.;  cksm1_in_prof2 = .false.;
	endif
    elseif(value == "CHECKSUM2 FLAG" .or. value == "CHECKSUM FLAG2")then
	hasCKSM(2) = .true.; ncksmcol(2) = icol						! Store checksum2 info
    elseif(value == "CHECKSUM3 FLAG" .or. value == "CHECKSUM FLAG3")then
	hasCKSM(3) = .true.; ncksmcol(3) = icol						! Store checksum3 info
    endif
    if(DOXY_float)then
	if(index(value,"2A") > 0) go to 200; ! SKIP 2A/B/C
	if(index(value,"2B") > 0) go to 200; ! SKIP 2A/B/C
	if(index(value,"2C") > 0) go to 200; ! SKIP 2A/B/C
	if(value(1: 9) == "TEMP_DOXY")then
	    ibpar = 1; hasBPAR(ibpar) = .true.; hasTEMP_DOXY(0) = .true.;
	    nbdx = nbdx + 1; bnamadnl(nbdx) = trim(BPARNAME(ibpar)); bcoladnl(nbdx) = icol; bparadnl(nbdx) = ibpar;
	    if(trim(value) == 'TEMP_DOXY (DEG C)')then
		hasTEMP_DOXY(1) = .true.; bprfadnl(nbdx) = 1;
	    elseif(trim(value) == 'TEMP_DOXY2 (DEG C)')then
		hasTEMP_DOXY(2) = .true.; bprfadnl(nbdx) = 2;
	    else
                ! We don;t recognized the name for this TEMP_DOXY variable
	

                ! We don;t recognized the name for this RPHASE variable
                write(errmsg(1),'("NON-FATAL ERROR 307 in read_phy, a TEMP_DOXY line was not identified")')
                write(errmsg(2),'("NON-FATAL ERROR 307 The line was: ",a)')trim(cline);
                write(errmsg(3),'("NON-FATAL ERROR 307 Processing will continue without this line")')
                call error_report(sub,307,3,errmsg);
	
	    endif
	    go to 200; ! Found this one, skip the rest
	endif
	if(value(1:14) == "FREQUENCY_DOXY")then
	    ibpar = 2; hasBPAR(ibpar) = .true.; hasFREQ_DOXY(0) = .true.; ! Note there are no FREQ_DOXY A/B/C
	    nbdx = nbdx + 1; bnamadnl(nbdx) = trim(BPARNAME(ibpar)); bcoladnl(nbdx) = icol; bparadnl(nbdx) = ibpar;
	    if(trim(value) == "FREQUENCY_DOXY")then
		hasFREQ_DOXY(1) = .true.; bprfadnl(nbdx) = 1;
	    elseif(trim(value) == "FREQUENCY_DOXY (HZ)")then
		hasFREQ_DOXY(1) = .true.; bprfadnl(nbdx) = 1;
	    elseif(trim(value) == "FREQUENCY_DOXY2 (HZ)")then
		hasFREQ_DOXY(2) = .true.; bprfadnl(nbdx) = 2;
	    else
		! We don;t recognized the name for this FREQUENCY_DOXY variable

                ! We don;t recognized the name for this RPHASE variable
                write(errmsg(1),'("NON-FATAL ERROR 308 in read_phy, a FREQUENCY_DOXY line was not identified")')
                write(errmsg(2),'("NON-FATAL ERROR 308 The line was: ",a)')trim(cline);
                write(errmsg(3),'("NON-FATAL ERROR 308 Processing will continue without this line")')
                call error_report(sub,308,3,errmsg);
	    endif
	    go to 200; ! Found this one, skip the rest

	endif
	if(value(1: 6) == "BPHASE")then
	    ibpar = 3; hasBPAR(ibpar) = .true.; hasBPHA_DOXY(0) = .true.; ! Note there are no BPHASE A/B/C
	    nbdx = nbdx + 1; bnamadnl(nbdx) = trim(BPARNAME(ibpar)); bcoladnl(nbdx) = icol; bparadnl(nbdx) = ibpar;
	    if(trim(value) == "BPHASE_DOXY")then
		hasBPHA_DOXY(1) = .true.; bprfadnl(nbdx) = 1;
	    elseif(trim(value) == "BPHASE_DOXY2")then
		hasBPHA_DOXY(2) = .true.; bprfadnl(nbdx) = 2;
	    else
		! We don;t recognized the name for this BPHASE variable
		write(errmsg(1),'("NON-FATAL ERROR 309 in read_phy for fle: ",a)')trim(phyfile);
		write(errmsg(2),'("NON-FATAL ERROR 309 PHASE value not recognized: ",a)')trim(value);
		write(errmsg(3),'("NON-FATAL ERROR 309 cline: ",a)')trim(cline);
		call error_report(sub,309,3,errmsg); ! read_phy PHASE value not recognized
	    endif
	    go to 200; ! Found this one, skip the rest
	endif

        if(value(1: 6) == "TPHASE")then
            ibpar = 4; hasBPAR(ibpar) = .true.; hasTPHA_DOXY(0) = .true.; ! Note there IS TPHASE A/B/C and plain TPHASE2A
	    nbdx = nbdx + 1; bnamadnl(nbdx) = trim(BPARNAME(ibpar)); bcoladnl(nbdx) = icol; bparadnl(nbdx) = ibpar;
            if(trim(value) == "TPHASE_DOXY")then
                hasTPHA_DOXY(1) = .true.; bprfadnl(nbdx) = 1;
            elseif(trim(value) == "TPHASE_DOXY2")then
                hasTPHA_DOXY(2) = .true.; bprfadnl(nbdx) = 2;
            else
                ! We don;t recognized the name for this TPHASE variable
		write(errmsg(1),'("NON-FATAL ERROR 310 in read_phy for fle: ",a)')trim(phyfile);
		write(errmsg(2),'("NON-FATAL ERROR 310 BPHASE value not recognized: ",a)')trim(value);
		write(errmsg(3),'("NON-FATAL ERROR 310 cline: ",a)')trim(cline);
		call error_report(sub,310,3,errmsg); ! read_phy BPHASE value not recognized
            endif
	    go to 200; ! Found this one, skip the rest
        endif

        if(value(1: 6) == "RPHASE")then
            ibpar = 5; hasBPAR(ibpar) = .true.; hasRPHA_DOXY(0) = .true.; ! Note there IS RPHASE A/B/C with&without _DOXY
            nbdx = nbdx + 1; bnamadnl(nbdx) = trim(BPARNAME(ibpar)); bcoladnl(nbdx) = icol; bparadnl(nbdx) = ibpar;
            if(trim(value) == "RPHASE_DOXY")then
                hasRPHA_DOXY(1) = .true.; bprfadnl(nbdx) = 1;
            elseif(trim(value) == "RPHASE_DOXY2")then
                hasRPHA_DOXY(2) = .true.; bprfadnl(nbdx) = 2;
	    elseif(trim(value) == "RPHASE")then
		hasRPHA_DOXY(1) = .true.; bprfadnl(nbdx) = 1;
	    elseif(trim(value) == "RPHASE2")then
		hasRPHA_DOXY(2) = .true.; bprfadnl(nbdx) = 2;
            else
                ! We don;t recognized the name for this RPHASE variable
		write(errmsg(1),'("NON-FATAL ERROR 311 in read_phy, a DOXY line was not identiried")')
		write(errmsg(2),'("NON-FATAL ERROR 311 The line was: ",a)')trim(cline);
		write(errmsg(3),'("NON-FATAL ERROR 311 Processing will continue without this line")')
		call error_report(sub,311,3,errmsg); 
            endif
	    go to 200; ! Found this one, skip the rest
        endif

	200 continue; ! SKIP 2A/B/C

	if(nbdx > 0)then
	    nadnlcols = nbdx; ! Save in a more descriptive name, not shorthand
	endif
	
    endif; ! DOXY_float
enddo

if(DOXY_float)then
    ! What is the nax number of profiles with DOXY data?
       if(hasDOXY(5))then; nbprf = 2; ! doxy2abc
    elseif(hasDOXY(4))then; nbprf = 2; ! doxy2ab
    elseif(hasDOXY(3))then; nbprf = 2; ! doxy2a
    elseif(hasDOXY(2))then; nbprf = 2;
    elseif(hasDOXY(1))then; nbprf = 1;
    endif
    ! Which additional doxy parameters are present, besides just pld DOXY
    nbparadnl = 0;
    if(hasTEMP_DOXY(0))then; nbparadnl=nbparadnl+1;  bparout(nbparadnl+2) = 'TEMP_DOXY'; endif    
    if(hasTEMP_DOXY(0))then;  write(msg,'("ERR INC @21 nbparadnl to",i2)')nbparadnl; call dbgmsg(sub,msg); endif
    if(hasFREQ_DOXY(0))then; nbparadnl=nbparadnl+1;  bparout(nbparadnl+2) = 'FREQUENCY_DOXY'; endif
    if(hasFREQ_DOXY(0))then;  write(msg,'("ERR INC @22 nbparadnl to",i2)')nbparadnl; call dbgmsg(sub,msg); endif
    if(hasBPHA_DOXY(0))then; nbparadnl=nbparadnl+1;  bparout(nbparadnl+2) = 'BPHASE_DOXY'; endif  
    if(hasBPHA_DOXY(0))then;  write(msg,'("ERR INC @23 nbparadnl to",i2)')nbparadnl; call dbgmsg(sub,msg); endif
    if(hasTPHA_DOXY(0))then; nbparadnl=nbparadnl+1;  bparout(nbparadnl+2) = 'TPHASE_DOXY'; endif  
    if(hasTPHA_DOXY(0))then;  write(msg,'("ERR INC @24 nbparadnl to",i2)')nbparadnl; call dbgmsg(sub,msg); endif
    if(hasRPHA_DOXY(0))then; nbparadnl=nbparadnl+1;  bparout(nbparadnl+2) = 'RPHASE_DOXY'; endif  
    if(hasRPHA_DOXY(0))then;  write(msg,'("ERR INC @25 nbparadnl to",i2)')nbparadnl; call dbgmsg(sub,msg); endif


endif 
    
! All header lines have been read, what data parameters do we have?

nprf = npres; ! How many parameters do we have?
if(npres > 0)then;
    hasPRES(0) = .true.; hasNCPAR(1) = .true.; hasQCPAR(1) = .true.;    ! PRES is ncparname(1) in NC file
    do i=1,MAXPRF; hasPARPRF(1,i) = hasPRES(i); enddo
endif
if(ntemp > 0)then;
    hasTEMP(0) = .true.; hasNCPAR(2) = .true.; hasQCPAR(2) = .true.;    ! TEMP is ncparname(2) in NC file
    do i=1,MAXPRF; hasPARPRF(2,i) = hasTEMP(i); enddo
endif
if(npsal > 0)then;
    hasPSAL(0) = .true.; hasNCPAR(4) = .true.; hasQCPAR(3) = .true.;    ! PSAL is ncparname(4) in NC file
    do i=1,MAXPRF; hasPARPRF(3,i) = hasPSAL(i); enddo
endif
if(ndoxy > 0)then;
    hasDOXY(0) = .true.; hasNCPAR(5) = .true.; hasQCPAR(4) = .true.;    ! DOXY is ncparname(5) in NC file
    do i=1,MAXPRF; hasPARPRF(4,i) = hasDOXY(i); enddo
endif
if(ncndc > 0)then;
    hasCNDC(0) = .true.; hasNCPAR(3) = .true.; hasQCPAR(5) = .true.;    ! CNDC is ncparname(3) in NC file
    do i=1,MAXPRF; hasPARPRF(5,i) = hasCNDC(i); enddo
endif

if(MAXPRF > 4)then
    write(errmsg(1),'("FATAL ERROR 916 MAXPRF changed, update read_phy")');
    write(errmsg(2),'("FATAL ERROR 916 change the hasDOXY kluge in read_phy")');
    call error_exit(sub,916,2,errmsg); ! read_phy change the hasDOXY kluge
endif

nprf = npres; ! How many parameters do we have?
if(npres > 0)then;
    hasPRES(0) = .true.; hasNCPAR(1) = .true.; hasQCPAR(1) = .true.;	! PRES is ncparname(1) in NC file
    do i=1,MAXPRF; hasPARPRF(1,i) = hasPRES(i); enddo
endif
if(ntemp > 0)then;
    hasTEMP(0) = .true.; hasNCPAR(2) = .true.; hasQCPAR(2) = .true.;	! TEMP is ncparname(2) in NC file
    do i=1,MAXPRF; hasPARPRF(2,i) = hasTEMP(i); enddo
endif
if(npsal > 0)then;
    hasPSAL(0) = .true.; hasNCPAR(4) = .true.; hasQCPAR(3) = .true.;	! PSAL is ncparname(4) in NC file
    do i=1,MAXPRF; hasPARPRF(3,i) = hasPSAL(i); enddo
endif
if(ndoxy > 0)then;
    hasDOXY(0) = .true.; hasNCPAR(5) = .true.; hasQCPAR(4) = .true.;	! DOXY is ncparname(5) in NC file
    do i=1,MAXPRF; hasPARPRF(4,i) = hasDOXY(i); enddo
endif
if(ncndc > 0)then;
    hasCNDC(0) = .true.; hasNCPAR(3) = .true.; hasQCPAR(5) = .true.;	! CNDC is ncparname(3) in NC file
    do i=1,MAXPRF; hasPARPRF(5,i) = hasCNDC(i); enddo
endif

if(DEBUG)then; write(msg,*)'npres: ',npres,' hasPRES: ',hasPRES; call wrtdbg(sub,msg); endif
if(DEBUG)then; write(msg,*)'ntemp: ',ntemp,' hasTEMP: ',hasTEMP; call wrtdbg(sub,msg); endif
if(DEBUG)then; write(msg,*)'npsal: ',npsal,' hasPSAL: ',hasPSAL; call wrtdbg(sub,msg); endif
if(DEBUG)then; write(msg,*)'ndoxy: ',ndoxy,' hasDOXY: ',hasDOXY; call wrtdbg(sub,msg); endif
if(DEBUG)then; write(msg,*)'ncndc: ',ncndc,' hasCNDC: ',hasCNDC; call wrtdbg(sub,msg); endif
if(DEBUG)then; write(msg,*)' nprf: ',nprf; call wrtdbg(sub,msg); endif
if(DEBUG)then; write(msg,*)'What is hasNCPAR: ',hasNCPAR; call wrtdbg(sub,msg); endif
if(hasPRES(0))then				! I knoq, we always have PRES
    do iprf=1,nprf
	if(hasPRES(iprf))then
	    hasDATA(regPRES(iprf)) = .true.
	endif
    enddo
endif
if(hasTEMP(0))then
    do iprf=1,nprf
	if(hasTEMP(iprf)) hasDATA(regTEMP(iprf)) = .true.
    enddo
endif
if(hasPSAL(0))then
    do iprf=1,nprf
	if(hasPSAL(iprf))then
	    hasDATA(regPSAL(iprf)) = .true.
	endif
    enddo
endif
if(hasDOXY(0))then
    do iprf=1,MAXDOXY
	if(hasDOXY(iprf)) hasDATA(regDOXY(iprf)) = .true.
    enddo
endif
if(nmc > 0)then;				! nmc = number of measurement code lines
    hasMC = .true.
    if(DBLEV==32)then;
	write(msg,'("LEV32 We have ",i5," MC lines")')nmc; call wrtdbg(sub,msg);
	do imc=1,nmc
	    write(msg,'(i4,x,a)')imc,trim(mcline(imc)); call wrtdbg(sub,msg);
	enddo
    endif
endif

if(DEBUG)then;
    if(do_pres_adj)then;
	write(msg,'("ADJUST PRES do_pres_adj is T surf_pres: ",f9.3)')surf_pres;
    else
	write(msg,'("ADJUST PRES do_pres_adj is F")');
    endif
    call wrtdbg(sub,msg);
endif
if(DEBUG)then;
    if(do_psal_adj)then;
	write(msg,'("ADJUST PSAL do_psal_adj is T psal_offset: ",f12.7)')psal_offset;
    else
	write(msg,'("ADJUST PSAL do_psal_adj is F")');
    endif
    call wrtdbg(sub,msg);
endif
if(DEBUG)then; write(msg,'("Number of inp columns: ",i3)')ncolinp; call wrtdbg(sub,msg); endif
HED=.false.; POS=.false.; DAT=.true.;  ! Finalloy, read the data

call initialize_arrays(sub);

if(DEBUG)then; write(msg,*)'Read the first dataline, check the number of columns'; call wrtdbg(sub,msg); endif
nlev = 0;
! Read the first data line
read(INP,'(a)',end=995,err=996,iostat=ios) cline; call check_inp(sub,phyfile,cline);
if(DBGIO)then; ninp=ninp+1; write(msg,'("INP DAT ",i6)')ninp; call dbgmsg(sub,msg); endif
read(cline,*,end=913,err=914)(datvals(i),i=1,ncolinp)
nlev = 1; ! This is the first data line
if(DBGIO)then; write(msg,'("DBGIO INP PHY lev:",i5,x,a)')nlev,trim(cline); call dbgmsg(sub,msg); endif


call parse_dataline(sub,nlev);	! Parse the first data line
datline(nlev) = trim(cline);
! Now, read the rest of the profile
do while(DAT)
    read(INP,'(a)',end=401,err=996,iostat=ios) cline; call check_inp(sub,phyfile,cline);
    if(DBGIO)then; ninp=ninp+1; write(msg,'("INP DAT ",i6)')ninp; call dbgmsg(sub,msg); endif
    nlev = nlev + 1; 
    if(nlev > MAXLEV)then
	write(errmsg(1),'("FATAL ERROR 507 in read_phy")')
	write(errmsg(2),'("FATAL ERROR 507 maximum number of levels exceds MAXLEV: ",i5)')MAXLEV
	call error_exit(sub,507,2,errmsg); ! read_phy maximum number of levels exceds MAXLEV
    endif
    if(DBGIO)then; write(msg,'("DBGIO INP PHY lev:",i5,x,a)')nlev,trim(cline); call dbgmsg(sub,msg); endif
    datline(nlev)=trim(cline)
    read(cline,*,end=913,err=914)(datvals(i),i=1,ncolinp)
    call parse_dataline(sub,nlev);	! Parse the dataline
enddo
401 continue ! End-of-file in data part
write(msg,*)'read_phy END OF DATA'; call wrtdbg(sub,msg);

! The earlier set of defs was the wrong setif(DOXY_float .and. nlev>0) go to 999; ! Skip this really bad section
if(DOXY_float .and. nlev>0)then
    ! NOTEST hasPRES(0) = .false.; ! TEST no PRES
    ! NOTEST hasDOXY(0) = .false.; ! TEST no DOXY

    ! For BR files, we "should" have two parameters always
    ! NOTEST hasPRES(0) = .false.; ! TEST
    if(.not. hasPRES(0))then
	write(errmsg(1),'("FATAL ERROR 508 parameter PRES no found in read_phy")'); ! ERR
	write(errmsg(2),'("FATAL ERROR 508 This float is identified as a DOXY float, but PRES is NOt PRESENT")'); ! ERR
	call error_exit(sub,50,2,errmsg); ! ERR DOXY float, but PRES not found
    endif
    ! NOTEST hasDOXY(0) = .false.; ! TEST
    if(.not. hasDOXY(0))then
	write(errmsg(1),'("FATAL ERROR 509 parameter DOXY not found in read_phy")'); ! ERR
	write(errmsg(2),'("FATAL ERROR 509 This float is identified as a DOXY float, but DOXY is NOt PRESENT")'); ! ERR
	call error_exit(sub,50,2,errmsg); ! ERR DOXY float but DOXY not found
    endif
    nbparout = 1; bparout(nbparout) = 'PRES'; ! We ALSAYS have PRES
    nbparout = 2; bparout(nbparout) = 'DOXY'; ! We ALWAYS have DOXY

    nbparout = nbparout + nbparadnl

endif; ! DOXY_float


999 continue; ! Skip  this really wrong part above

! DBG write(DBG,'("JOB_ST ZERO+ ??? There MAY BE zerolines, nzerolines= ",i4," nliv: ",i4)')nzerolines,nlev;

if(zerolines)then;
    if(nzerolines == nlev)then
	write(msg,'("ZEROLINES zero+ There ARE zerolines, nzerolines==nlev ALL levels are zerolines: ",i4)')nzerolines; 
    else
	write(msg,'("ZEROLINES zero+ There ARE zerolines, the number is: ",i3)')nzerolines;
    endif
else
    write(msg,'("ZEROLINES zero- There are NO zerolines")');
endif
call wrtlog(sub,msg);

if(DEBUG)then
    write(msg,'("INFO ",a20,i5)')profile,nlev
    if(hasDATA(p1r)) write(msg,'(a,x,"P1")')trim(msg)
    if(hasDATA(t1r)) write(msg,'(a,x,"T1")')trim(msg)
    if(hasDATA(s1r)) write(msg,'(a,x,"S1")')trim(msg)
    if(hasDATA(p2r)) write(msg,'(a,x,"P2")')trim(msg)
    if(hasDATA(t2r)) write(msg,'(a,x,"T2")')trim(msg)
    if(hasDATA(s2r)) write(msg,'(a,x,"S2")')trim(msg)
    if(hasDATA(p3r)) write(msg,'(a,x,"P3")')trim(msg)
    if(hasDATA(t3r)) write(msg,'(a,x,"T3")')trim(msg)
    if(hasDATA(s3r)) write(msg,'(a,x,"S3")')trim(msg)
    if(hasDATA(p4r)) write(msg,'(a,x,"P4")')trim(msg)
    if(hasDATA(t4r)) write(msg,'(a,x,"T4")')trim(msg)
    if(hasDATA(s4r)) write(msg,'(a,x,"S4")')trim(msg)
    if(hasDOXY(1))     write(msg,'(a,x,"O1")')trim(msg)
    if(hasDOXY(2))     write(msg,'(a,x,"O2")')trim(msg)
    if(hasDOXY(3))     write(msg,'(a,x,"OA")')trim(msg)
    if(hasDOXY(4))     write(msg,'(a,x,"OB")')trim(msg)
    if(hasDOXY(5))     write(msg,'(a,x,"OC")')trim(msg)
    if(hasCNDC(1))   write(msg,'(a,x,"C1")')trim(msg)
    if(hasCNDC(2))   write(msg,'(a,x,"C2")')trim(msg)
    if(hasCNDC(3))   write(msg,'(a,x,"C3")')trim(msg)
    if(hasCNDC(4))   write(msg,'(a,x,"C4")')trim(msg)
    call wrtdbg(sub,msg);					! DBG
endif

if(nlev == 1) OneDAT = .true.

! See if the data need to be reversed,so surface is at top
!
! 60    ARGOS communications, sampling on up transit
! 61    ARGOS communications, sampling on down transit
! 62    Orbcomm communications, sampling on up transit
! 63    Orbcomm communications, sampling on down transit
! 64    Iridium communications, sampling on up transit
! 65    Iridium communications, sampling on down transit
!
! EXCEPT, on the deep argo floats, John has the wrong recorder type (64( but has
! already reversed the profile to reflect this - i.e., he had the highest press
! at the top, so two wongs make a right

! KEEP the preliminary msg def - is used in normal messaging, not just testing
write(msg,'("By WMO_RECORDER_TYPE (",a2,") this profile is ")')chr_WMO_RECORDER_TYPE; call wrttrc(sub,msg);
if(chr_WMO_RECORDER_TYPE =='60' .or. chr_WMO_RECORDER_TYPE=='62' .or. chr_WMO_RECORDER_TYPE =='64')then
    write(msg,'(a," ASC so it DOES need to be reversed")')trim(msg); call wrtlog(sub,msg);
    direction_WMO_rec = 'A';	! This is an ASC float
else
    direction_WMO_rec = 'D';    ! This is an DSC float
endif

! summarize the direction clues
write(direction_summary,'(4a1)')direction_filename,direction_configdir,direction_DOPs,direction_WMO_rec
if(DBGDIR) write(DBG,'("DBGDIR read_phy direction_filename : ",a)') direction_filename
if(DBGDIR) write(DBG,'("DBGDIR read_phy direction_configdir: ",a)') direction_configdir
if(DBGDIR) write(DBG,'("DBGDIR read_phy direction_DOPs     : ",a)') direction_DOPs
if(DBGDIR) write(DBG,'("DBGDIR read_phy direction_WMO_rec  : ",a)') direction_WMO_rec

if(DEBUG)then
    write(msg,'("TEST DIRECTION       phy_descending: ",l1)')phy_descending;        call wrtdbg(sub,msg); ! DBG
    write(msg,'("TEST DIRECTION   direction_filename: ",a1)')direction_filename;    call wrtdbg(sub,msg); ! DBG
    write(msg,'("TEST DIRECTION   direction_cnfgdir : ",a1)')direction_configdir;   call wrtdbg(sub,msg); ! DBG
    write(msg,'("TEST DIRECTION       direction_DOPs: ",a1)')direction_DOPs;        call wrtdbg(sub,msg); ! DBG
    write(msg,'("TEST DIRECTION    direction_WMO_rec: ",a1)')direction_WMO_rec;     call wrtdbg(sub,msg); ! DBG
    write(msg,'("TEST DIRECTION              Summary: ",a)')direction_summary;      call wrtdbg(sub,msg); ! DBG
endif

if(DBGDIR) write(DBG,'("DBGDIR read_phy    phy_descending: ",l1)') phy_descending
if(DBGDIR) write(DBG,'("DBGDIR read_phy direction_summary: ",a)')  direction_summary
if(DBGDIR) write(DBG,'("DBGDIR read_phy -------------------------")');

! At least one case is setting phy_dewcenting to T due to direction_WMO_rec, but ! direction_filename (* others) would set to F


if(((index(direction_summary,"A") >0)) .and. (index(direction_summary,"D") >0))then
    write(errmsg(1),'("NON-FATAL ERROR 312 in read_phy for fle: ",a)')trim(phyfile);
    if((direction_filename == "D") .and. (direction_WMO_rec == "A"))then
        write(errmsg(2),'("NON-FATAL ERROR 312 direction_filename is D and direction_WMO_rec is A")');
        write(errmsg(3),'("NON-FATAL ERROR 312 the direction_WMO_rec must be WRONG")');
        call error_report(sub,312,3,errmsg);
        if(DBGDIR) write(DBG,'("DBGDIR read_phy NON-FATAL ERROR 312 direction_filename is D and direction_WMO_rec is A")')
    else
        write(errmsg(1),'("NON-FATAL ERROR 312 in read_phy for fle: ",a)')trim(phyfile);
        if(DBGDIR) write(DBG,'("DBGDIR read_phy ERR1: ",a)')trim(errmsg(1));
    endif
endif

if(DEBUG)then; write(msg,'("TEST DIRECTION Summary: ",a)')direction_summary; call wrtdbg(sub,msg); endif
if(.not. phy_descending)then
    write(msg,'("This is an ASC float, so the profile must be reversed")')
    call wrtlog(sub,msg);
    call reverse_profdata(sub)
    profile_reversed = .true.
endif

call set_proftopbottot(sub)

if(DBGIO)then;										! DBGIO
    write(msg,'("DBGIO INP After reading datafile, nlev: ",i5)')nlev; call wrtdbg(sub,msg);	! DBGIO
    call dbg_levs(sub,1,5,"First five")								! DBGIO
    if(nlev > 4)then;										! DBGIO
	call dbg_levs(sub,nlev-4,nlev," Last ive")						! DBGIO
    else											! DBGIO
	write(msg,'("There are only ",i5," levels")')nlev					! DBGIO
    endif											! DBGIO
endif												! DBGIO
read_phy = 0;			! Signal success

inp_open=close_file(INP,"INP","pyy file");

if(hasTEMP_DOXY(0))then; do iprf=1,2; write(msg,2004)1,iprf,(profbdata(1,iprf,ilev),ilev=1,10); call dbgmsg(sub,msg); enddo; endif;
if(hasFREQ_DOXY(0))then; do iprf=1,2; write(msg,2004)2,iprf,(profbdata(2,iprf,ilev),ilev=1,10); call dbgmsg(sub,msg); enddo; endif;
if(hasBPHA_DOXY(0))then; do iprf=1,2; write(msg,2004)3,iprf,(profbdata(3,iprf,ilev),ilev=1,10); call dbgmsg(sub,msg); enddo; endif;
if(hasTPHA_DOXY(0))then; do iprf=1,2; write(msg,2004)4,iprf,(profbdata(4,iprf,ilev),ilev=1,10); call dbgmsg(sub,msg); enddo; endif;
if(hasRPHA_DOXY(0))then; do iprf=1,2; write(msg,2004)5,iprf,(profbdata(5,iprf,ilev),ilev=1,10); call dbgmsg(sub,msg); enddo; endif;
2004 format("BRD(rp) par/prf/lev: ",2i3,10f10.2)

if(DBGIO) call dbg_leave(sub,read_phy);

return; ! Normal return

! ============================== ERROR HANDLING ==============================

989 continue
    write(errmsg(1),'("FATAL ERROR 501 unlabeled position block in phy file at 989")');
    write(errmsg(2),'("FATAL ERROR 501 trim(phyfile)")');
    write(errmsg(3),'("FATAL ERROR 501 no LAT/LON/YEAR/MM/DD line after first =========== line")');
    call error_exit(sub,501,3,errmsg); ! read_phy unlabeled position block
990 continue
    write(errmsg(1),'("FATAL ERROR 502 incomplete HED of phy file at 990")');
    write(errmsg(2),'("FATAL ERROR 502 trim(phyfile)")');
    write(errmsg(3),'("FATAL ERROR 502 end_of_file before header complete")');
    call error_exit(sub,502,3,errmsg); ! read_phy incomplete HED
991 continue
    write(errmsg(1),'("FATAL ERROR 503 read statement reported err HED of phy file")');
    write(errmsg(2),'("FATAL ERROR 503 trim(phyfile)")');
    write(errmsg(3),'("FATAL ERROR 503 read statement reported error ios: ",i2)')ios;
    call error_exit(sub,503,3,errmsg); ! read_phy read statement reported err HED
992 continue
    write(errmsg(1),'("FATAL ERROR 504 incomplete POS of phy file at 992")');
    write(errmsg(2),'("FATAL ERROR 504 trim(phyfile)")');
    write(errmsg(3),'("FATAL ERROR 504 end_of_file before position block complete")');
    call error_exit(sub,504,3,errmsg); ! read_phy incomplete POS
993 continue
    write(errmsg(1),'("FATAL ERROR 505 error reading POS of phy file at 993")');
    write(errmsg(2),'("FATAL ERROR 505 trim(phyfile)")');
    write(errmsg(3),'("FATAL ERROR 505 read reported error ios: ",i2)')ios;
    call error_exit(sub,505,3,errmsg); ! read_phy error reading POS
995 continue
    NoDAT = .true.
    read_phy = 0; ! Signal "success" (ie, file read, no info on nlev/NoHED,NoPOS,NoDAT)
    return
996 continue
    write(errmsg(1),'("FATAL ERROR 506 error reading data lines at 996")');
    write(errmsg(2),'("FATAL ERROR 506 read error iostat: ",i2)') ios
    write(errmsg(3),'("FATAL ERROR 506 read error on line: ",i4)') nlev
    call error_exit(sub,506,3,errmsg); ! read_phy error reading data lines
913 continue
    write(errmsg(1),'("FATAL ERROR 913 error reading datavals from input line")');
    write(errmsg(2),'("FATAL ERROR 913 DATVALS end detected")');
    write(errmsg(3),'("FATAL ERROR 913 DATVALS the number of columns is not correct")');
    write(errmsg(4),'("FATAL ERROR 913 ",a)')trim(cline);
    call error_exit(sub,913,4,errmsg); ! read_phy error reading datavals
914 continue
    write(errmsg(1),'("FATAL ERROR 914 error reading dataals from input line")');
    write(errmsg(2),*)'FATAL ERROR 914 DATVALS error detected';
    write(errmsg(3),*)'FATAL ERROR 914 DATVALS the number of columns is not correct';
    call error_exit(sub,914,3,errmsg); ! read_phy error reading dataals

end function read_phy

!=========================================================================
subroutine parse_apexsbd_config_dbar(val,outval)

use communication                       ! module
use data_arrays                         ! module

implicit none

character(*), intent( in ) :: val
character(*), intent( out) :: outval

integer :: ncm		! number of commas
integer,parameter :: LEVMAX = 3;
integer          :: binsiz(LEVMAX), binsep(LEVMAX-1)	!  int layer information
character(len=10) :: cbinsiz, cbinsep			! char layer information
character(len=5)  :: c5, csiz(LEVMAX), csep(LEVMAX-1)	! char output line
integer :: lsc,lval;                                    ! locatin of the semicolon, length of val
character(len=216) :: tmpout				! temporary outpout string
!="Primary sampling: bin-averaged[nominal ";
!=" dbar binned data 0-";
!=" dbar, ";
!=" dbar binned data "

lsc=index(val,";");
lval = len(trim(val));
cbinsiz = val(1:lsc-1);
cbinsep = val(lsc+1:lval);

read(cbinsiz,*)binsiz;
do i=1,LEVMAX;
    write(c5,'(i5)')binsiz(i);
    csiz(i) = adjustl(c5)
enddo
read(cbinsep,*)binsep;
do i=1,LEVMAX-1;
    write(c5,'(i5)')binsep(i);
    csep(i) = adjustl(c5);
enddo

tmpout = "Primary sampling: bin-averaged[nominal " // &
         trim(csiz(1)) // " dbar binned data 0-" // &
         trim(csep(1)) // " dbar, " // &
         trim(csiz(2)) // " dbar binned data " // &
         trim(csep(1)) // "-" // &
         trim(csep(2)) // "  dbar, " // &
         trim(csiz(3)) // " dbar binned data below " // &
         trim(csep(2)) // " dbar]"

outval = tmpout;

return

end subroutine parse_apexsbd_config_dbar
! =========================================================================================================== 
